# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# stdlib
from functools import partial

# mixbox
from mixbox import fields
from mixbox import entities
from mixbox import typedlist

# stix
import stix
import stix.bindings.stix_common as stix_common_binding


def validate_value(instance, value):
    allowed = instance._ALLOWED_VALUES

    if not value:
        return
    elif not allowed:
        return
    elif value in allowed:
        return
    else:
        error = "Value must be one of {allowed}. Received '{value}'"
        error = error.format(**locals())
        raise ValueError(error)

class VocabList(typedlist.TypedList):
    """VocabString fields can be any type of VocabString, though there is often
    a preferred/default VocabString type.

    The TypedList will attempt to make sure that every input item is an instance
    of the default VocabString and throw an error if it isn't. This sublcass
    overrides that behavior and allows any instance of VocabString to be
    inserted.
    """

    def _is_valid(self, value):
        return isinstance(value, VocabString)


class VocabField(fields.TypedField):
    """TypedField subclass for VocabString fields."""

    def __init__(self, *args, **kwargs):
        """Intercepts the __init__() call to TypedField.

        Set the type that will be used in from_dict and from_obj calls to
        :class:`VocabString`.

        Set the type that will be used in ``__set__`` for casting as the
        original ``type_`` argument, or :class:`VocabString` if no `type_`
        argument was provided.

        """
        super(VocabField, self).__init__(*args, **kwargs)
        self.factory = VocabFactory  # force this factory

        if self._unresolved_type is None:
            self.type_ = VocabString

        self._listfunc = partial(VocabList, type=self._unresolved_type)

    def check_type(self, value):
        return isinstance(value, VocabString)


class VocabFactory(entities.EntityFactory):
    _convert_strings = True
    
    @classmethod
    def entity_class(cls, key):
        try:
            return stix.lookup_extension(key, default=VocabString)
        except ValueError:
            return VocabString


class VocabString(stix.Entity):
    _binding = stix_common_binding
    _binding_class = stix_common_binding.ControlledVocabularyStringType
    _namespace = 'http://stix.mitre.org/common-1'

    # All subclasses should override these
    _XSI_TYPE = None
    _ALLOWED_VALUES = None

    value = fields.TypedField("valueOf_", key_name="value", preset_hook=validate_value)
    vocab_name = fields.TypedField("vocab_name")
    vocab_reference = fields.TypedField("vocab_reference")
    xsi_type = fields.TypedField("xsi_type", key_name="xsi:type")

    def __init__(self, value=None):
        super(VocabString, self).__init__()
        self.value = value
        self.xsi_type = self._XSI_TYPE

    def __str__(self):
        return str(self.value)

    def __eq__(self, other):
        # Check to make sure the values are identical.
        if isinstance(other, VocabString):
            other = other.value

        return other == self.value

    def is_plain(self):
        """Whether the VocabString can be represented as a single value."""
        return (
            self.xsi_type is None and
            self.vocab_name is None and
            self.vocab_reference is None
        )

    def to_dict(self):
        if self.is_plain():
            return self.value
        return super(VocabString, self).to_dict()


    @classmethod
    def from_dict(cls, cls_dict):
        if not cls_dict:
            vocab =  None
        elif not isinstance(cls_dict, dict):
            vocab = cls()
            vocab.value = cls_dict
        else:
            vocab = super(VocabString, cls).from_dict(cls_dict)

        return vocab


def _get_terms(vocab_class):
    """Helper function used by register_vocab."""
    for k, v in vars(vocab_class).items():
        if k.startswith("TERM_"):
            yield v


def add_vocab(cls):
    """Registers a VocabString subclass.

    Note:
        The :meth:`register_vocab` class decorator has replaced this method.

    """
    stix.add_extension(cls)


def register_vocab(cls):
    """Class decorator that registers a VocabString subclass.

    Also, calculate all the permitted values for class being decorated by
    adding an ``_ALLOWED_VALUES`` tuple of all the values of class members
    beginning with ``TERM_``.

    """
    add_vocab(cls)

    cls._ALLOWED_VALUES = tuple(_get_terms(cls))
    return cls


@register_vocab
class AvailabilityLossType_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:AvailabilityLossTypeVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_ACCELERATION = 'Acceleration'
    TERM_DEGREDATION = 'Degredation'
    TERM_DESTRUCTION = 'Destruction'
    TERM_INTERRUPTION = 'Interruption'
    TERM_LOSS = 'Loss'
    TERM_OBSCURATION = 'Obscuration'
    TERM_UNKNOWN = 'Unknown'


@register_vocab
class AvailabilityLossType_1_1_1(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:AvailabilityLossTypeVocab-1.1.1'
    _VOCAB_VERSION = '1.1.1'

    TERM_ACCELERATION = 'Acceleration'
    TERM_DEGRADATION = 'Degradation'
    TERM_DESTRUCTION = 'Destruction'
    TERM_INTERRUPTION = 'Interruption'
    TERM_LOSS = 'Loss'
    TERM_OBSCURATION = 'Obscuration'
    TERM_UNKNOWN = 'Unknown'


@register_vocab
class ThreatActorType_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:ThreatActorTypeVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_CYBER_ESPIONAGE_OPERATIONS = "Cyber Espionage Operations"
    TERM_DISGRUNTLED_CUSTOMER_OR_USER = "Disgruntled Customer / User"
    TERM_ECRIME_ACTOR_CREDENTIAL_THEFT_BOTNET_OPERATOR = "eCrime Actor - Credential Theft Botnet Operator"
    TERM_ECRIME_ACTOR_CREDENTIAL_THEFT_BOTNET_SERVICE = "eCrime Actor - Credential Theft Botnet Service"
    TERM_ECRIME_ACTOR_MALWARE_DEVELOPER = "eCrime Actor - Malware Developer"
    TERM_ECRIME_ACTOR_MONEY_LAUNDERING_NETWORK = "eCrime Actor - Money Laundering Network"
    TERM_ECRIME_ACTOR_ORGANIZED_CRIME_ACTOR = "eCrime Actor - Organized Crime Actor"
    TERM_ECRIME_ACTOR_SPAM_SERVICE = "eCrime Actor - Spam Service"
    TERM_ECRIME_ACTOR_TRAFFIC_SERVICE = "eCrime Actor - Traffic Service"
    TERM_ECRIME_ACTOR_UNDERGROUND_CALL_SERVICE = "eCrime Actor - Underground Call Service"
    TERM_HACKER = "Hacker"
    TERM_HACKER_BLACK_HAT = "Hacker - Black hat"
    TERM_HACKER_GRAY_HAT = "Hacker - Gray hat"
    TERM_HACKER_WHITE_HAT = "Hacker - White hat"
    TERM_HACKTIVIST = "Hacktivist"
    TERM_INSIDER_THREAT = "Insider Threat"
    TERM_STATE_ACTOR_OR_AGENCY = "State Actor / Agency"


@register_vocab
class AttackerInfrastructureType_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:AttackerInfrastructureTypeVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_ANONYMIZATION = "Anonymization"
    TERM_ANONYMIZATION_PROXY = "Anonymization - Proxy"
    TERM_ANONYMIZATION_TOR_NETWORK = "Anonymization - TOR Network"
    TERM_ANONYMIZATION_VPN = "Anonymization - VPN"
    TERM_COMMUNICATIONS = "Communications"
    TERM_COMMUNICATIONS_BLOGS = "Communications - Blogs"
    TERM_COMMUNICATIONS_FORUMS = "Communications - Forums"
    TERM_COMMUNICATIONS_INTERNET_RELAY_CHAT = "Communications - Internet Relay Chat"
    TERM_COMMUNICATIONS_MICROBLOGS = "Communications - Micro-Blogs"
    TERM_COMMUNICATIONS_MOBILE_COMMUNICATIONS = "Communications - Mobile Communications"
    TERM_COMMUNICATIONS_SOCIAL_NETWORKS = "Communications - Social Networks"
    TERM_COMMUNICATIONS_USERGENERATED_CONTENT_WEBSITES = "Communications - User-Generated Content Websites"
    TERM_DOMAIN_REGISTRATION = "Domain Registration"
    TERM_DOMAIN_REGISTRATION_DYNAMIC_DNS_SERVICES = "Domain Registration - Dynamic DNS Services"
    TERM_DOMAIN_REGISTRATION_LEGITIMATE_DOMAIN_REGISTRATION_SERVICES = "Domain Registration - Legitimate Domain Registration Services"
    TERM_DOMAIN_REGISTRATION_MALICIOUS_DOMAIN_REGISTRARS = "Domain Registration - Malicious Domain Registrars"
    TERM_DOMAIN_REGISTRATION_TOPLEVEL_DOMAIN_REGISTRARS = "Domain Registration - Top-Level Domain Registrars"
    TERM_ELECTRONIC_PAYMENT_METHODS = "Electronic Payment Methods"
    TERM_HOSTING = "Hosting"
    TERM_HOSTING_BULLETPROOF_OR_ROGUE_HOSTING = "Hosting - Bulletproof / Rogue Hosting"
    TERM_HOSTING_CLOUD_HOSTING = "Hosting - Cloud Hosting"
    TERM_HOSTING_COMPROMISED_SERVER = "Hosting - Compromised Server"
    TERM_HOSTING_FAST_FLUX_BOTNET_HOSTING = "Hosting - Fast Flux Botnet Hosting"
    TERM_HOSTING_LEGITIMATE_HOSTING = "Hosting - Legitimate Hosting"


@register_vocab
class DiscoveryMethod_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:DiscoveryMethodVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_AGENT_DISCLOSURE = "Agent Disclosure"
    TERM_ANTIVIRUS = "Antivirus"
    TERM_AUDIT = "Audit"
    TERM_CUSTOMER = "Customer"
    TERM_FINANCIAL_AUDIT = "Financial Audit"
    TERM_FRAUD_DETECTION = "Fraud Detection"
    TERM_HIPS = "HIPS"
    TERM_INCIDENT_RESPONSE = "Incident Response"
    TERM_IT_AUDIT = "IT Audit"
    TERM_LAW_ENFORCEMENT = "Law Enforcement"
    TERM_LOG_REVIEW = "Log Review"
    TERM_MONITORING_SERVICE = "Monitoring Service"
    TERM_NIDS = "NIDS"
    TERM_SECURITY_ALARM = "Security Alarm"
    TERM_UNKNOWN = "Unknown"
    TERM_UNRELATED_PARTY = "Unrelated Party"
    TERM_USER = "User"

@register_vocab
class DiscoveryMethod_2_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:DiscoveryMethodVocab-2.0'
    _VOCAB_VERSION = '2.0'

    TERM_AGENT_DISCLOSURE = "Agent Disclosure"
    TERM_ANTIVIRUS = "Antivirus"
    TERM_AUDIT = "Audit"
    TERM_CUSTOMER = "Customer"
    TERM_EXTERNAL_FRAUD_DETECTION = "External - Fraud Detection"
    TERM_FINANCIAL_AUDIT = "Financial Audit"
    TERM_HIPS = "HIPS"
    TERM_INCIDENT_RESPONSE = "Incident Response"
    TERM_INTERNAL_FRAUD_DETECTION = "Internal - Fraud Detection"
    TERM_IT_AUDIT = "IT Audit"
    TERM_LAW_ENFORCEMENT = "Law Enforcement"
    TERM_LOG_REVIEW = "Log Review"
    TERM_MONITORING_SERVICE = "Monitoring Service"
    TERM_NIDS = "NIDS"
    TERM_SECURITY_ALARM = "Security Alarm"
    TERM_UNKNOWN = "Unknown"
    TERM_UNRELATED_PARTY = "Unrelated Party"
    TERM_USER = "User"


@register_vocab
class AttackerToolType_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:AttackerToolTypeVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_APPLICATION_SCANNER = "Application Scanner"
    TERM_MALWARE = "Malware"
    TERM_PASSWORD_CRACKING = "Password Cracking"
    TERM_PENETRATION_TESTING = "Penetration Testing"
    TERM_PORT_SCANNER = "Port Scanner"
    TERM_TRAFFIC_SCANNER = "Traffic Scanner"
    TERM_VULNERABILITY_SCANNER = "Vulnerability Scanner"


@register_vocab
class IndicatorType_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:IndicatorTypeVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_ANONYMIZATION = 'Anonymization'
    TERM_C2 = 'C2'
    TERM_DOMAIN_WATCHLIST = 'Domain Watchlist'
    TERM_EXFILTRATION = 'Exfiltration'
    TERM_FILE_HASH_WATCHLIST = 'File Hash Watchlist'
    TERM_HOST_CHARACTERISTICS = 'Host Characteristics'
    TERM_IP_WATCHLIST = 'IP Watchlist'
    TERM_MALICIOUS_EMAIL = 'Malicious E-mail'
    TERM_MALWARE_ARTIFACTS = 'Malware Artifacts'
    TERM_URL_WATCHLIST = 'URL Watchlist'


@register_vocab
class IndicatorType_1_1(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:IndicatorTypeVocab-1.1'
    _VOCAB_VERSION = '1.1'

    TERM_ANONYMIZATION = 'Anonymization'
    TERM_C2 = 'C2'
    TERM_COMPROMISED_PKI_CERTIFICATE = 'Compromised PKI Certificate'
    TERM_DOMAIN_WATCHLIST = 'Domain Watchlist'
    TERM_EXFILTRATION = 'Exfiltration'
    TERM_FILE_HASH_WATCHLIST = 'File Hash Watchlist'
    TERM_HOST_CHARACTERISTICS = 'Host Characteristics'
    TERM_IMEI_WATCHLIST = 'IMEI Watchlist'
    TERM_IMSI_WATCHLIST = 'IMSI Watchlist'
    TERM_IP_WATCHLIST = 'IP Watchlist'
    TERM_LOGIN_NAME = 'Login Name'
    TERM_MALICIOUS_EMAIL = 'Malicious E-mail'
    TERM_MALWARE_ARTIFACTS = 'Malware Artifacts'
    TERM_URL_WATCHLIST = 'URL Watchlist'


@register_vocab
class SystemType_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:SystemTypeVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_ENTERPRISE_SYSTEMS = "Enterprise Systems"
    TERM_ENTERPRISE_SYSTEMS_APPLICATION_LAYER = "Enterprise Systems - Application Layer"
    TERM_ENTERPRISE_SYSTEMS_DATABASE_LAYER = "Enterprise Systems - Database Layer"
    TERM_ENTERPRISE_SYSTEMS_ENTERPRISE_TECHNOLOGIES_AND_SUPPORT_INFRASTRUCTURE = "Enterprise Systems - Enterprise Technologies and Support Infrastructure"
    TERM_ENTERPRISE_SYSTEMS_NETWORKING_DEVICES = "Enterprise Systems - Networking Devices"
    TERM_ENTERPRISE_SYSTEMS_NETWORK_SYSTEMS = "Enterprise Systems - Network Systems"
    TERM_ENTERPRISE_SYSTEMS_VOIP = "Enterprise Systems - VoIP"
    TERM_ENTERPRISE_SYSTEMS_WEB_LAYER = "Enterprise Systems - Web Layer"
    TERM_INDUSTRIAL_CONTROL_SYSTEMS = "Industrial Control Systems"
    TERM_INDUSTRIAL_CONTROL_SYSTEMS_EQUIPMENT_UNDER_CONTROL = "Industrial Control Systems - Equipment Under Control"
    TERM_INDUSTRIAL_CONTROL_SYSTEMS_OPERATIONS_MANAGEMENT = "Industrial Control Systems - Operations Management"
    TERM_INDUSTRIAL_CONTROL_SYSTEMS_SAFETY_PROTECTION_AND_LOCAL_CONTROL = "Industrial Control Systems - Safety, Protection and Local Control"
    TERM_INDUSTRIAL_CONTROL_SYSTEMS_SUPERVISORY_CONTROL = "Industrial Control Systems - Supervisory Control"
    TERM_MOBILE_SYSTEMS = "Mobile Systems"
    TERM_MOBILE_SYSTEMS_MOBILE_DEVICES = "Mobile Systems - Mobile Devices"
    TERM_MOBILE_SYSTEMS_MOBILE_OPERATING_SYSTEMS = "Mobile Systems - Mobile Operating Systems"
    TERM_MOBILE_SYSTEMS_NEAR_FIELD_COMMUNICATIONS = "Mobile Systems - Near Field Communications"
    TERM_THIRDPARTY_SERVICES = "Third-Party Services"
    TERM_THIRDPARTY_SERVICES_APPLICATION_STORES = "Third-Party Services - Application Stores"
    TERM_THIRDPARTY_SERVICES_CLOUD_SERVICES = "Third-Party Services - Cloud Services"
    TERM_THIRDPARTY_SERVICES_SECURITY_VENDORS = "Third-Party Services - Security Vendors"
    TERM_THIRDPARTY_SERVICES_SOCIAL_MEDIA = "Third-Party Services - Social Media"
    TERM_THIRDPARTY_SERVICES_SOFTWARE_UPDATE = "Third-Party Services - Software Update"
    TERM_USERS = "Users"
    TERM_USERS_APPLICATION_AND_SOFTWARE = "Users - Application And Software"
    TERM_USERS_REMOVABLE_MEDIA = "Users - Removable Media"
    TERM_USERS_WORKSTATION = "Users - Workstation"


@register_vocab
class CampaignStatus_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:CampaignStatusVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_FUTURE = "Future"
    TERM_HISTORIC = "Historic"
    TERM_ONGOING = "Ongoing"


@register_vocab
class IncidentStatus_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:IncidentStatusVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_CLOSED = "Closed"
    TERM_CONTAINMENT_ACHIEVED = "Containment Achieved"
    TERM_DELETED = "Deleted"
    TERM_INCIDENT_REPORTED = "Incident Reported"
    TERM_NEW = "New"
    TERM_OPEN = "Open"
    TERM_REJECTED = "Rejected"
    TERM_RESTORATION_ACHIEVED = "Restoration Achieved"
    TERM_STALLED = "Stalled"


@register_vocab
class ManagementClass_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:ManagementClassVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_COMANAGEMENT = "Co-Management"
    TERM_EXTERNALLYMANAGEMENT = "Externally-Management"
    TERM_INTERNALLYMANAGED = "Internally-Managed"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class Motivation_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:MotivationVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_EGO = 'Ego'
    TERM_FINANCIAL_OR_ECONOMIC = 'Financial or Economic'
    TERM_IDEOLOGICAL = 'Ideological'
    TERM_IDEOLOGICAL_ANTICORRUPTION = 'Ideological - Anti-Corruption'
    TERM_IDEOLOGICAL_ANTIESTABLISMENT = 'Ideological - Anti-Establisment'
    TERM_IDEOLOGICAL_ENVIRONMENTAL = 'Ideological - Environmental'
    TERM_IDEOLOGICAL_ETHNIC_NATIONALIST = 'Ideological - Ethnic / Nationalist'
    TERM_IDEOLOGICAL_HUMAN_RIGHTS = 'Ideological - Human Rights'
    TERM_IDEOLOGICAL_INFORMATION_FREEDOM = 'Ideological - Information Freedom'
    TERM_IDEOLOGICAL_RELIGIOUS = 'Ideological - Religious'
    TERM_IDEOLOGICAL_SECURITY_AWARENESS = 'Ideological - Security Awareness'
    TERM_MILITARY = 'Military'
    TERM_OPPORTUNISTIC = 'Opportunistic'
    TERM_POLICITAL = 'Policital'


@register_vocab
class Motivation_1_0_1(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:MotivationVocab-1.0.1'
    _VOCAB_VERSION = '1.0.1'

    TERM_EGO = 'Ego'
    TERM_FINANCIAL_OR_ECONOMIC = 'Financial or Economic'
    TERM_IDEOLOGICAL = 'Ideological'
    TERM_IDEOLOGICAL_ANTI_CORRUPTION = 'Ideological - Anti-Corruption'
    TERM_IDEOLOGICAL_ANTI_ESTABLISHMENT = 'Ideological - Anti-Establishment'
    TERM_IDEOLOGICAL_ENVIRONMENTAL = 'Ideological - Environmental'
    TERM_IDEOLOGICAL_ETHNIC_NATIONALIST = 'Ideological - Ethnic / Nationalist'
    TERM_IDEOLOGICAL_HUMAN_RIGHTS = 'Ideological - Human Rights'
    TERM_IDEOLOGICAL_INFORMATION_FREEDOM = 'Ideological - Information Freedom'
    TERM_IDEOLOGICAL_SECURITY_AWARENESS = 'Ideological - Security Awareness'
    TERM_IDEOLOGICAL__RELIGIOUS = 'Ideological - Religious'
    TERM_MILITARY = 'Military'
    TERM_OPPORTUNISTIC = 'Opportunistic'
    TERM_POLICITAL = 'Policital'


@register_vocab
class Motivation_1_1(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:MotivationVocab-1.1'
    _VOCAB_VERSION = '1.1'

    TERM_EGO = 'Ego'
    TERM_FINANCIAL_OR_ECONOMIC = 'Financial or Economic'
    TERM_IDEOLOGICAL = 'Ideological'
    TERM_IDEOLOGICAL_ANTICORRUPTION = 'Ideological - Anti-Corruption'
    TERM_IDEOLOGICAL_ANTIESTABLISHMENT = 'Ideological - Anti-Establishment'
    TERM_IDEOLOGICAL_ENVIRONMENTAL = 'Ideological - Environmental'
    TERM_IDEOLOGICAL_ETHNIC_NATIONALIST = 'Ideological - Ethnic / Nationalist'
    TERM_IDEOLOGICAL_HUMAN_RIGHTS = 'Ideological - Human Rights'
    TERM_IDEOLOGICAL_INFORMATION_FREEDOM = 'Ideological - Information Freedom'
    TERM_IDEOLOGICAL_RELIGIOUS = 'Ideological - Religious'
    TERM_IDEOLOGICAL_SECURITY_AWARENESS = 'Ideological - Security Awareness'
    TERM_MILITARY = 'Military'
    TERM_OPPORTUNISTIC = 'Opportunistic'
    TERM_POLITICAL = 'Political'


@register_vocab
class IncidentCategory_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:IncidentCategoryVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_DENIAL_OF_SERVICE = "Denial of Service"
    TERM_EXERCISEORNETWORK_DEFENSE_TESTING = "Exercise/Network Defense Testing"
    TERM_IMPROPER_USAGE = "Improper Usage"
    TERM_INVESTIGATION = "Investigation"
    TERM_MALICIOUS_CODE = "Malicious Code"
    TERM_SCANSORPROBESORATTEMPTED_ACCESS = "Scans/Probes/Attempted Access"
    TERM_UNAUTHORIZED_ACCESS = "Unauthorized Access"


@register_vocab
class ImpactQualification_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:ImpactQualificationVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_CATASTROPHIC = "Catastrophic"
    TERM_DAMAGING = "Damaging"
    TERM_DISTRACTING = "Distracting"
    TERM_INSIGNIFICANT = "Insignificant"
    TERM_PAINFUL = "Painful"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class PlanningAndOperationalSupport_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:PlanningAndOperationalSupportVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_DATA_EXPLOITATION = 'Data Exploitation'
    TERM_DATA_EXPLOITATION_ANALYTIC_SUPPORT = 'Data Exploitation - Analytic Support'
    TERM_DATA_EXPLOITATION_TRANSLATION_SUPPORT = 'Data Exploitation - Translation Support'
    TERM_FINANCIAL_RESOURCES = 'Financial Resources'
    TERM_FINANCIAL_RESOURCES_ACADEMIC = 'Financial Resources - Academic'
    TERM_FINANCIAL_RESOURCES_COMMERCIAL = 'Financial Resources - Commercial'
    TERM_FINANCIAL_RESOURCES_GOVERNMENT = 'Financial Resources - Government'
    TERM_FINANCIAL_RESOURCES_HACKTIVIST_OR_GRASSROOT = 'Financial Resources - Hacktivist or Grassroot'
    TERM_FINANCIAL_RESOURCES_NONATTRIBUTABLE_FINANCE = 'Financial Resources - Non-Attributable Finance'
    TERM_PLANNING = 'Planning '
    TERM_PLANNING_OPEN_SOURCE_INTELLIGENCE_OSINT_GETHERING = 'Planning - Open-Source Intelligence (OSINT) Gethering'
    TERM_PLANNING_OPERATIONAL_COVER_PLAN = 'Planning - Operational Cover Plan'
    TERM_PLANNING_PRE_OPERATIONAL_SURVEILLANCE_AND_RECONNAISSANCE = 'Planning - Pre-Operational Surveillance and Reconnaissance'
    TERM_PLANNING_TARGET_SELECTION = 'Planning - Target Selection'
    TERM_SKILL_DEVELOPMENT_RECRUITMENT = 'Skill Development / Recruitment'
    TERM_SKILL_DEVELOPMENT_RECRUITMENT_CONTRACTING_AND_HIRING = 'Skill Development / Recruitment - Contracting and Hiring'
    TERM_SKILL_DEVELOPMENT_RECRUITMENT_DOCUMENT_EXPLOITATION_DOCEX_TRAINING = 'Skill Development / Recruitment - Document Exploitation (DOCEX) Training'
    TERM_SKILL_DEVELOPMENT_RECRUITMENT_INTERNAL_TRAINING = 'Skill Development / Recruitment - Internal Training'
    TERM_SKILL_DEVELOPMENT_RECRUITMENT_MILITARY_PROGRAMS = 'Skill Development / Recruitment - Military Programs'
    TERM_SKILL_DEVELOPMENT_RECRUITMENT_SECURITY_HACKER_CONFERENCES = 'Skill Development / Recruitment - Security / Hacker Conferences'
    TERM_SKILL_DEVELOPMENT_RECRUITMENT_UNDERGROUND_FORUMS = 'Skill Development / Recruitment - Underground Forums'
    TERM_SKILL_DEVELOPMENT_RECRUITMENT_UNIVERSITY_PROGRAMS = 'Skill Development / Recruitment - University Programs'


@register_vocab
class PlanningAndOperationalSupport_1_0_1(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:PlanningAndOperationalSupportVocab-1.0.1'
    _VOCAB_VERSION = '1.0.1'

    TERM_DATA_EXPLOITATION = "Data Exploitation"
    TERM_DATA_EXPLOITATION_ANALYTIC_SUPPORT = "Data Exploitation - Analytic Support"
    TERM_DATA_EXPLOITATION_TRANSLATION_SUPPORT = "Data Exploitation - Translation Support"
    TERM_FINANCIAL_RESOURCES = "Financial Resources"
    TERM_FINANCIAL_RESOURCES_ACADEMIC = "Financial Resources - Academic"
    TERM_FINANCIAL_RESOURCES_COMMERCIAL = "Financial Resources - Commercial"
    TERM_FINANCIAL_RESOURCES_GOVERNMENT = "Financial Resources - Government"
    TERM_FINANCIAL_RESOURCES_HACKTIVIST_OR_GRASSROOT = "Financial Resources - Hacktivist or Grassroot"
    TERM_FINANCIAL_RESOURCES_NONATTRIBUTABLE_FINANCE = "Financial Resources - Non-Attributable Finance"
    TERM_PLANNING = "Planning"
    TERM_PLANNING_OPENSOURCE_INTELLIGENCE_OSINT_GATHERING = "Planning - Open-Source Intelligence (OSINT) Gathering"
    TERM_PLANNING_OPERATIONAL_COVER_PLAN = "Planning - Operational Cover Plan"
    TERM_PLANNING_PREOPERATIONAL_SURVEILLANCE_AND_RECONNAISSANCE = "Planning - Pre-Operational Surveillance and Reconnaissance"
    TERM_PLANNING_TARGET_SELECTION = "Planning - Target Selection"
    TERM_SKILL_DEVELOPMENT_OR_RECRUITMENT = "Skill Development / Recruitment"
    TERM_SKILL_DEVELOPMENT_OR_RECRUITMENT_CONTRACTING_AND_HIRING = "Skill Development / Recruitment - Contracting and Hiring"
    TERM_SKILL_DEVELOPMENT_OR_RECRUITMENT_DOCUMENT_EXPLOITATION_DOCEX_TRAINING = "Skill Development / Recruitment - Document Exploitation (DOCEX) Training"
    TERM_SKILL_DEVELOPMENT_OR_RECRUITMENT_INTERNAL_TRAINING = "Skill Development / Recruitment - Internal Training"
    TERM_SKILL_DEVELOPMENT_OR_RECRUITMENT_MILITARY_PROGRAMS = "Skill Development / Recruitment - Military Programs"
    TERM_SKILL_DEVELOPMENT_OR_RECRUITMENT_SECURITY_OR_HACKER_CONFERENCES = "Skill Development / Recruitment - Security / Hacker Conferences"
    TERM_SKILL_DEVELOPMENT_OR_RECRUITMENT_UNDERGROUND_FORUMS = "Skill Development / Recruitment - Underground Forums"
    TERM_SKILL_DEVELOPMENT_OR_RECRUITMENT_UNIVERSITY_PROGRAMS = "Skill Development / Recruitment - University Programs"


@register_vocab
class CourseOfActionType_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:CourseOfActionTypeVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_DIPLOMATIC_ACTIONS = "Diplomatic Actions"
    TERM_ERADICATION = "Eradication"
    TERM_HARDENING = "Hardening"
    TERM_INTERNAL_BLOCKING = "Internal Blocking"
    TERM_LOGICAL_ACCESS_RESTRICTIONS = "Logical Access Restrictions"
    TERM_MONITORING = "Monitoring"
    TERM_OTHER = "Other"
    TERM_PATCHING = "Patching"
    TERM_PERIMETER_BLOCKING = "Perimeter Blocking"
    TERM_PHYSICAL_ACCESS_RESTRICTIONS = "Physical Access Restrictions"
    TERM_POLICY_ACTIONS = "Policy Actions"
    TERM_PUBLIC_DISCLOSURE = "Public Disclosure"
    TERM_REBUILDING = "Rebuilding"
    TERM_REDIRECTION = "Redirection"
    TERM_REDIRECTION_HONEY_POT = "Redirection (Honey Pot)"
    TERM_TRAINING = "Training"


@register_vocab
class SecurityCompromise_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:SecurityCompromiseVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_NO = "No"
    TERM_SUSPECTED = "Suspected"
    TERM_UNKNOWN = "Unknown"
    TERM_YES = "Yes"


@register_vocab
class ImpactRating_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:ImpactRatingVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_MAJOR = "Major"
    TERM_MINOR = "Minor"
    TERM_MODERATE = "Moderate"
    TERM_NONE = "None"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class AssetType_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:AssetTypeVocab-1.0'
    _VOCAB_VERSION = '1.0'


    TERM_ACCESS_READER = "Access reader"
    TERM_ADMINISTRATOR = "Administrator"
    TERM_ATM = "ATM"
    TERM_AUDITOR = "Auditor"
    TERM_AUTH_TOKEN = "Auth token"
    TERM_BACKUP = "Backup"
    TERM_BROADBAND = "Broadband"
    TERM_CALL_CENTER = "Call center"
    TERM_CAMERA = "Camera"
    TERM_CASHIER = "Cashier"
    TERM_CUSTOMER = "Customer"
    TERM_DATABASE = "Database"
    TERM_DCS = "DCS"
    TERM_DESKTOP = "Desktop"
    TERM_DEVELOPER = "Developer"
    TERM_DHCP = "DHCP"
    TERM_DIRECTORY = "Directory"
    TERM_DISK_DRIVE = "Disk drive"
    TERM_DISK_MEDIA = "Disk media"
    TERM_DNS = "DNS"
    TERM_DOCUMENTS = "Documents"
    TERM_ENDUSER = "End-user"
    TERM_EXECUTIVE = "Executive"
    TERM_FILE = "File"
    TERM_FINANCE = "Finance"
    TERM_FIREWALL = "Firewall"
    TERM_FLASH_DRIVE = "Flash drive"
    TERM_FORMER_EMPLOYEE = "Former employee"
    TERM_GAS_TERMINAL = "Gas terminal"
    TERM_GUARD = "Guard"
    TERM_HELPDESK = "Helpdesk"
    TERM_HSM = "HSM"
    TERM_HUMAN_RESOURCES = "Human resources"
    TERM_IDS = "IDS"
    TERM_KIOSK = "Kiosk"
    TERM_LAN = "LAN"
    TERM_LAPTOP = "Laptop"
    TERM_LOG = "Log"
    TERM_MAIL = "Mail"
    TERM_MAINFRAME = "Mainframe"
    TERM_MAINTENANCE = "Maintenance"
    TERM_MANAGER = "Manager"
    TERM_MEDIA = "Media"
    TERM_MOBILE_PHONE = "Mobile phone"
    TERM_NETWORK = "Network"
    TERM_PARTNER = "Partner"
    TERM_PAYMENT_CARD = "Payment card"
    TERM_PAYMENT_SWITCH = "Payment switch"
    TERM_PBX = "PBX"
    TERM_PED_PAD = "PED pad"
    TERM_PERIPHERAL = "Peripheral"
    TERM_PERSON = "Person"
    TERM_PLC = "PLC"
    TERM_POS_CONTROLLER = "POS controller"
    TERM_POS_TERMINAL = "POS terminal"
    TERM_PRINT = "Print"
    TERM_PRIVATE_WAN = "Private WAN"
    TERM_PROXY = "Proxy"
    TERM_PUBLIC_WAN = "Public WAN"
    TERM_REMOTE_ACCESS = "Remote access"
    TERM_ROUTER_OR_SWITCH = "Router or switch"
    TERM_RTU = "RTU"
    TERM_SAN = "SAN"
    TERM_SCADA = "SCADA"
    TERM_SERVER = "Server"
    TERM_SMART_CARD = "Smart card"
    TERM_TABLET = "Tablet"
    TERM_TAPES = "Tapes"
    TERM_TELEPHONE = "Telephone"
    TERM_UNKNOWN = "Unknown"
    TERM_USER_DEVICE = "User Device"
    TERM_VOIP_ADAPTER = "VoIP adapter"
    TERM_VOIP_PHONE = "VoIP phone"
    TERM_WEB_APPLICATION = "Web application"
    TERM_WLAN = "WLAN"

@register_vocab
class COAStage_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:COAStageVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_REMEDY = "Remedy"
    TERM_RESPONSE = "Response"


@register_vocab
class LocationClass_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:LocationClassVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_COLOCATED = "Co-Located"
    TERM_EXTERNALLYLOCATED = "Externally-Located"
    TERM_INTERNALLYLOCATED = "Internally-Located"
    TERM_MOBILE = "Mobile"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class InformationType_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:InformationTypeVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_AUTHENTICATION_COOKIES = "Authentication Cookies"
    TERM_INFORMATION_ASSETS = "Information Assets"
    TERM_INFORMATION_ASSETS_CORPORATE_EMPLOYEE_INFORMATION = "Information Assets - Corporate Employee Information"
    TERM_INFORMATION_ASSETS_CUSTOMER_PII = "Information Assets - Customer PII"
    TERM_INFORMATION_ASSETS_EMAIL_LISTS_OR_ARCHIVES = "Information Assets - Email Lists / Archives"
    TERM_INFORMATION_ASSETS_FINANCIAL_DATA = "Information Assets - Financial Data"
    TERM_INFORMATION_ASSETS_INTELLECTUAL_PROPERTY = "Information Assets - Intellectual Property"
    TERM_INFORMATION_ASSETS_MOBILE_PHONE_CONTACTS = "Information Assets - Mobile Phone Contacts"
    TERM_INFORMATION_ASSETS_USER_CREDENTIALS = "Information Assets - User Credentials"


@register_vocab
class ThreatActorSophistication_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:ThreatActorSophisticationVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_ASPIRANT = "Aspirant"
    TERM_EXPERT = "Expert"
    TERM_INNOVATOR = "Innovator"
    TERM_NOVICE = "Novice"
    TERM_PRACTITIONER = "Practitioner"


@register_vocab
class HighMediumLow_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:HighMediumLowVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_HIGH = "High"
    TERM_LOW = "Low"
    TERM_MEDIUM = "Medium"
    TERM_NONE = "None"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class LossProperty_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:LossPropertyVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_ACCOUNTABILITY = "Accountability"
    TERM_AVAILABILITY = "Availability"
    TERM_CONFIDENTIALITY = "Confidentiality"
    TERM_INTEGRITY = "Integrity"
    TERM_NONREPUDIATION = "Non-Repudiation"


@register_vocab
class IntendedEffect_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:IntendedEffectVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_ACCOUNT_TAKEOVER = "Account Takeover"
    TERM_ADVANTAGE = "Advantage"
    TERM_ADVANTAGE_ECONOMIC = "Advantage - Economic"
    TERM_ADVANTAGE_MILITARY = "Advantage - Military"
    TERM_ADVANTAGE_POLITICAL = "Advantage - Political"
    TERM_BRAND_DAMAGE = "Brand Damage"
    TERM_COMPETITIVE_ADVANTAGE = "Competitive Advantage"
    TERM_DEGRADATION_OF_SERVICE = "Degradation of Service"
    TERM_DENIAL_AND_DECEPTION = "Denial and Deception"
    TERM_DESTRUCTION = "Destruction"
    TERM_DISRUPTION = "Disruption"
    TERM_EMBARRASSMENT = "Embarrassment"
    TERM_EXPOSURE = "Exposure"
    TERM_EXTORTION = "Extortion"
    TERM_FRAUD = "Fraud"
    TERM_HARASSMENT = "Harassment"
    TERM_ICS_CONTROL = "ICS Control"
    TERM_THEFT = "Theft"
    TERM_THEFT_CREDENTIAL_THEFT = "Theft - Credential Theft"
    TERM_THEFT_IDENTITY_THEFT = "Theft - Identity Theft"
    TERM_THEFT_INTELLECTUAL_PROPERTY = "Theft - Intellectual Property"
    TERM_THEFT_THEFT_OF_PROPRIETARY_INFORMATION = "Theft - Theft of Proprietary Information"
    TERM_TRAFFIC_DIVERSION = "Traffic Diversion"
    TERM_UNAUTHORIZED_ACCESS = "Unauthorized Access"


@register_vocab
class PackageIntent_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:PackageIntentVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_ATTACK_PATTERN_CHARACTERIZATION = "Attack Pattern Characterization"
    TERM_CAMPAIGN_CHARACTERIZATION = "Campaign Characterization"
    TERM_COLLECTIVE_THREAT_INTELLIGENCE = "Collective Threat Intelligence"
    TERM_COURSES_OF_ACTION = "Courses of Action"
    TERM_EXPLOIT_CHARACTERIZATION = "Exploit Characterization"
    TERM_INCIDENT = "Incident"
    TERM_INDICATORS = "Indicators"
    TERM_INDICATORS_ENDPOINT_CHARACTERISTICS = "Indicators - Endpoint Characteristics"
    TERM_INDICATORS_MALWARE_ARTIFACTS = "Indicators - Malware Artifacts"
    TERM_INDICATORS_NETWORK_ACTIVITY = "Indicators - Network Activity"
    TERM_INDICATORS_PHISHING = "Indicators - Phishing"
    TERM_INDICATORS_WATCHLIST = "Indicators - Watchlist"
    TERM_MALWARE_CHARACTERIZATION = "Malware Characterization"
    TERM_MALWARE_SAMPLES = "Malware Samples"
    TERM_OBSERVATIONS = "Observations"
    TERM_OBSERVATIONS_EMAIL = "Observations - Email"
    TERM_THREAT_ACTOR_CHARACTERIZATION = "Threat Actor Characterization"
    TERM_THREAT_REPORT = "Threat Report"
    TERM_TTP_INFRASTRUCTURE = "TTP - Infrastructure"
    TERM_TTP_TOOLS = "TTP - Tools"


@register_vocab
class ReportIntent_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:ReportIntentVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_ATTACK_PATTERN_CHARACTERIZATION = "Attack Pattern Characterization"
    TERM_CAMPAIGN_CHARACTERIZATION = "Campaign Characterization"
    TERM_COLLECTIVE_THREAT_INTELLIGENCE = "Collective Threat Intelligence"
    TERM_COURSES_OF_ACTION = "Courses of Action"
    TERM_EXPLOIT_CHARACTERIZATION = "Exploit Characterization"
    TERM_INCIDENT = "Incident"
    TERM_INDICATORS = "Indicators"
    TERM_INDICATORS_ENDPOINT_CHARACTERISTICS = "Indicators - Endpoint Characteristics"
    TERM_INDICATORS_MALWARE_ARTIFACTS = "Indicators - Malware Artifacts"
    TERM_INDICATORS_NETWORK_ACTIVITY = "Indicators - Network Activity"
    TERM_INDICATORS_PHISHING = "Indicators - Phishing"
    TERM_INDICATORS_WATCHLIST = "Indicators - Watchlist"
    TERM_MALWARE_CHARACTERIZATION = "Malware Characterization"
    TERM_MALWARE_SAMPLES = "Malware Samples"
    TERM_OBSERVATIONS = "Observations"
    TERM_OBSERVATIONS_EMAIL = "Observations - Email"
    TERM_THREAT_ACTOR_CHARACTERIZATION = "Threat Actor Characterization"
    TERM_THREAT_REPORT = "Threat Report"
    TERM_TTP_INFRASTRUCTURE = "TTP - Infrastructure"
    TERM_TTP_TOOLS = "TTP - Tools"


@register_vocab
class LossDuration_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:LossDurationVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_DAYS = "Days"
    TERM_HOURS = "Hours"
    TERM_MINUTES = "Minutes"
    TERM_PERMANENT = "Permanent"
    TERM_SECONDS = "Seconds"
    TERM_UNKNOWN = "Unknown"
    TERM_WEEKS = "Weeks"


@register_vocab
class OwnershipClass_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:OwnershipClassVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_CUSTOMEROWNED = "Customer-Owned"
    TERM_EMPLOYEEOWNED = "Employee-Owned"
    TERM_INTERNALLYOWNED = "Internally-Owned"
    TERM_PARTNEROWNED = "Partner-Owned"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class MalwareType_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:MalwareTypeVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_ADWARE = "Adware"
    TERM_AUTOMATED_TRANSFER_SCRIPTS = "Automated Transfer Scripts"
    TERM_BOT = "Bot"
    TERM_BOT_CREDENTIAL_THEFT = "Bot - Credential Theft"
    TERM_BOT_DDOS = "Bot - DDoS"
    TERM_BOT_LOADER = "Bot - Loader"
    TERM_BOT_SPAM = "Bot - Spam"
    TERM_DIALER = "Dialer"
    TERM_DOS_OR_DDOS = "DoS / DDoS"
    TERM_DOS_OR_DDOS_PARTICIPATORY = "DoS / DDoS - Participatory"
    TERM_DOS_OR_DDOS_SCRIPT = "DoS / DDoS - Script"
    TERM_DOS_OR_DDOS_STRESS_TEST_TOOLS = "DoS / DDoS - Stress Test Tools"
    TERM_EXPLOIT_KITS = "Exploit Kits"
    TERM_POS_OR_ATM_MALWARE = "POS / ATM Malware"
    TERM_RANSOMWARE = "Ransomware"
    TERM_REMOTE_ACCESS_TROJAN = "Remote Access Trojan"
    TERM_ROGUE_ANTIVIRUS = "Rogue Antivirus"
    TERM_ROOTKIT = "Rootkit"


@register_vocab
class IncidentEffect_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:IncidentEffectVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_BRAND_OR_IMAGE_DEGRADATION = "Brand or Image Degradation"
    TERM_DATA_BREACH_OR_COMPROMISE = "Data Breach or Compromise"
    TERM_DEGRADATION_OF_SERVICE = "Degradation of Service"
    TERM_DESTRUCTION = "Destruction"
    TERM_DISRUPTION_OF_SERVICE_OR_OPERATIONS = "Disruption of Service / Operations"
    TERM_FINANCIAL_LOSS = "Financial Loss"
    TERM_LOSS_OF_COMPETITIVE_ADVANTAGE = "Loss of Competitive Advantage"
    TERM_LOSS_OF_COMPETITIVE_ADVANTAGE_ECONOMIC = "Loss of Competitive Advantage - Economic"
    TERM_LOSS_OF_COMPETITIVE_ADVANTAGE_MILITARY = "Loss of Competitive Advantage - Military"
    TERM_LOSS_OF_COMPETITIVE_ADVANTAGE_POLITICAL = "Loss of Competitive Advantage - Political"
    TERM_LOSS_OF_CONFIDENTIAL_OR_PROPRIETARY_INFORMATION_OR_INTELLECTUAL_PROPERTY = "Loss of Confidential / Proprietary Information or Intellectual Property"
    TERM_REGULATORY_COMPLIANCE_OR_LEGAL_IMPACT = "Regulatory, Compliance or Legal Impact"
    TERM_UNINTENDED_ACCESS = "Unintended Access"
    TERM_USER_DATA_LOSS = "User Data Loss"


@register_vocab
class InformationSourceRole_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:InformationSourceRoleVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_AGGREGATOR = "Aggregator"
    TERM_CONTENT_ENHANCERORREFINER = "Content Enhancer/Refiner"
    TERM_INITIAL_AUTHOR = "Initial Author"
    TERM_TRANSFORMERORTRANSLATOR = "Transformer/Translator"


@register_vocab
class Versioning_1_0(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:VersioningVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_REVOKES = "Revokes"
    TERM_UPDATES_REVISES = "Updates - Revises"
    TERM_UPDATE_CORRECTS = "Updates - Corrects"


# Vocab aliases which resolve to the most recent version of the
# associated VocabString.
AssetType = AssetType_1_0
AttackerInfrastructureType = AttackerInfrastructureType_1_0
AttackerToolType = AttackerToolType_1_0
AvailabilityLossType = AvailabilityLossType_1_1_1
CampaignStatus = CampaignStatus_1_0
COAStage = COAStage_1_0
CourseOfActionType = CourseOfActionType_1_0
DiscoveryMethod = DiscoveryMethod_2_0
HighMediumLow = HighMediumLow_1_0
ImpactQualification = ImpactQualification_1_0
ImpactRating = ImpactRating_1_0
IncidentCategory = IncidentCategory_1_0
IncidentEffect = IncidentEffect_1_0
IncidentStatus = IncidentStatus_1_0
IndicatorType = IndicatorType_1_1
InformationSourceRole = InformationSourceRole_1_0
InformationType = InformationType_1_0
IntendedEffect = IntendedEffect_1_0
LocationClass = LocationClass_1_0
LossDuration = LossDuration_1_0
LossProperty = LossProperty_1_0
MalwareType = MalwareType_1_0
ManagementClass = ManagementClass_1_0
Motivation = Motivation_1_1
OwnershipClass = OwnershipClass_1_0
PackageIntent = PackageIntent_1_0
PlanningAndOperationalSupport = PlanningAndOperationalSupport_1_0_1
ReportIntent = ReportIntent_1_0
SecurityCompromise = SecurityCompromise_1_0
SystemType = SystemType_1_0
ThreatActorSophistication = ThreatActorSophistication_1_0
ThreatActorType = ThreatActorType_1_0
Versioning = Versioning_1_0



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# stdlib
from distutils.version import LooseVersion

# external
from lxml import etree
import mixbox.xml
from mixbox.vendor.six import BytesIO, iteritems, binary_type

# internal
import stix
import stix.utils as utils
import stix.ttp.malware_instance
from stix.ttp.malware_instance import MalwareInstance
import stix.bindings.extensions.malware.maec_4_1 as ext_binding
from mixbox import fields
from stix.bindings.extensions.malware.maec_4_1 import maec_installed
from lxml.etree import _ElementTree

_MIN_PYTHON_MAEC_VERSION = '4.1.0.12'


class UnsupportedVersion(Exception):
    def __init__(self, message, expected, found):
        super(UnsupportedVersion, self).__init__(message)
        self.expected = expected
        self.found = found


def _check_maec_version():
    """Checks that the installed python-maec has a version greater than or
    equal to the minimum supported version.

    Note:
        We do this rather than having a python-maec dependency requirement
        listed in setup.py because MAEC is used as an extension to STIX and
        not a core component to STIX (like CybOX).

    Raises:
        ImportError: If python-maec is not installed.
        UnsupportedVersion: If the python-maec installation does not satisfy
            the version requirements.

    """
    import maec

    found = maec.__version__
    expected = _MIN_PYTHON_MAEC_VERSION

    if LooseVersion(found) >= LooseVersion(expected):
        return

    fmt = ("Unsupported python-maec version installed: '%s'. Minimum version "
           "is '%s'.")
    error = fmt % (found, expected)
    raise UnsupportedVersion(error, expected=expected, found=found)


try:
    # Check that the correct version of python-maec is installed.
    _check_maec_version()

    # Import maecPackage into global space
    from maec.package.package import Package as maecPackage

    _MAEC_INSTALLED = True
except ImportError:
    maecPackage, Package = None, None
    _MAEC_INSTALLED = False


def is_maec(obj):
    """Checks if the input object is python-maec object.

    Returns:
        True if python-maec is ins

    """
    if not _MAEC_INSTALLED:
        return False

    return isinstance(obj, maecPackage)

def validate_maec_input(instance, value):
    if value is None:
        return
    elif _MAEC_INSTALLED and is_maec(value):
        return
    elif mixbox.xml.is_element(value) or mixbox.xml.is_etree(value):
        return
    else:
        error = (
            "Cannot set maec to '{0}'. Expected 'lxml.etree._Element' or "
            "'maec.package.package.Package'."
        )
        error = error.format(type(value))
        raise ValueError(error)

@stix.register_extension
class MAECInstance(MalwareInstance):
    _binding = ext_binding
    _binding_class = _binding.MAEC4_1InstanceType
    _namespace = 'http://stix.mitre.org/extensions/Malware#MAEC4.1-1'
    _xml_ns_prefix = "stix-maec"
    _XSI_TYPE = "stix-maec:MAEC4.1InstanceType"
    _TAG_MAEC = "{%s}MAEC" % _namespace

    maec = fields.TypedField("MAEC", preset_hook=validate_maec_input)

    def __init__(self, maec=None):
        super(MAECInstance, self).__init__()
        self.__input_namespaces__ = {}
        self.__input_schemalocations__ = {}
        self.maec = maec

    def _parse_etree(self, root):
        node_tag = root.tag

        if node_tag != self._TAG_MAEC:
            self._cast_maec(root)

        self._collect_namespaces(root)
        self._collect_schemalocs(root)

    def _cast_maec(self, node):
        ns_maec = "http://maec.mitre.org/XMLSchema/maec-package-2"
        node_ns = etree.QName(node).namespace

        if node_ns == ns_maec:
            etree.register_namespace(self._xml_ns_prefix, self._namespace)
            node.tag = self._TAG_MAEC
        else:
            error = "Cannot set maec. Expected tag '{0}' found '{1}'."
            error = error.format(self._TAG_MAEC, node.tag)
            raise ValueError(error)

    def _collect_schemalocs(self, node):
        try:
            schemaloc = mixbox.xml.get_schemaloc_pairs(node)
            self.__input_schemalocations__ = dict(schemaloc)
        except KeyError:
            self.__input_schemalocations__ = {}

    def _collect_namespaces(self, node):
        self.__input_namespaces__ = dict(iteritems(node.nsmap))

    @classmethod
    def from_obj(cls, obj):
        if not obj:
            return None

        return_obj = cls()

        if _MAEC_INSTALLED:
            obj.MAEC = maecPackage.from_obj(obj.MAEC)
        else:
            obj.MAEC = obj.MAEC

        return_obj = super(MAECInstance, cls).from_obj(obj)

        return return_obj

    def to_obj(self, ns_info=None):
        return_obj = super(MAECInstance, self).to_obj(ns_info=ns_info)

        if mixbox.xml.is_element(self.maec) or mixbox.xml.is_etree(self.maec):
            tree = mixbox.xml.get_etree(self.maec)
            root = mixbox.xml.get_etree_root(tree)
            self._parse_etree(root)
            self.maec = root

        if _MAEC_INSTALLED and isinstance(self.maec, maecPackage):
            return_obj.MAEC = self.maec.to_obj(ns_info=ns_info)
        else:
            return_obj.MAEC = self.maec

        return return_obj

    @classmethod
    def _maec_from_dict(cls, d):
        if _MAEC_INSTALLED:
            return maecPackage.from_dict(d)

        raise ValueError(
            "Unable to parse 'maec' value in dictionary. Please "
            "install python-maec to parse dictionary value."
        )

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None

        d = d.copy()

        maec = d.get('maec')

        if maec is None:
            pass
        elif isinstance(maec, dict):
            d['maec'] = cls._maec_from_dict(maec)
        elif isinstance(maec, binary_type):
            d['maec'] = mixbox.xml.get_etree_root(BytesIO(maec))
        else:
            raise TypeError("Unknown type for 'maec' entry.")

        return_obj = super(MAECInstance, cls).from_dict(d)

        return return_obj

    def to_dict(self):
        d = super(MAECInstance, self).to_dict()

        if self.maec is not None:
            if mixbox.xml.is_element(self.maec) or mixbox.xml.is_etree(self.maec):
                tree = mixbox.xml.get_etree(self.maec)
                root = mixbox.xml.get_etree_root(tree)
                self._parse_etree(root)
                self.maec = root

            if _MAEC_INSTALLED and isinstance(self.maec, maecPackage):
                d['maec'] = self.maec.to_dict()
            else:
                d['maec'] = etree.tostring(self.maec)

            if self._XSI_TYPE:
                d['xsi:type'] = self._XSI_TYPE

        return d



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import stix
import stix.utils
import stix.indicator.test_mechanism
from stix.common import EncodedCDATA
from stix.indicator.test_mechanism import _BaseTestMechanism
import stix.bindings.extensions.test_mechanism.yara as yara_tm_binding


@stix.register_extension
class YaraTestMechanism(_BaseTestMechanism):
    _namespace = "http://stix.mitre.org/extensions/TestMechanism#YARA-1"
    _binding = yara_tm_binding
    _binding_class = _binding.YaraTestMechanismType
    _XSI_TYPE = "yaraTM:YaraTestMechanismType"

    version = fields.TypedField("Version")
    rule = fields.TypedField("Rule", EncodedCDATA)

    def __init__(self, id_=None, idref=None):
        super(YaraTestMechanism, self).__init__(id_=id_, idref=idref)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from __future__ import absolute_import
from sys import version_info

from mixbox.fields import TypedField, CDATAField

from .structured_text import StructuredText, StructuredTextList  # noqa
from .vocabs import VocabString   # noqa
from .datetimewithprecision import DateTimeWithPrecision  # noqa
from .activity import Activity  # noqa
from .confidence import Confidence  # noqa
from .identity import Identity  # noqa
from .information_source import InformationSource  # noqa
from .statement import Statement  # noqa
from .tools import ToolInformation  # noqa
from .names import Names  # noqa
from .campaign_reference import CampaignRef  # noqa
from .references import References  # noqa
from .profiles import Profiles  # noqa
from .related import (   # noqa
    GenericRelationshipList, RelatedCampaign, RelatedCOA,
    RelatedExploitTarget, RelatedIdentity, RelatedIncident,
    RelatedIndicator, RelatedObservable, RelatedThreatActor, RelatedTTP,
    RelatedPackage, RelatedPackages, RelatedCampaignRef, RelatedReports,
    RelatedReport
)

import stix
import stix.utils as utils
import stix.bindings.stix_common as common_binding

from mixbox.vendor.six import text_type

class EncodedCDATA(stix.Entity):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = _binding.EncodedCDATAType

    value = CDATAField("valueOf_", key_name="value")
    encoded = TypedField("encoded")

    def __init__(self, value=None, encoded=None):
        super(EncodedCDATA, self).__init__()
        self.value = value
        self.encoded = encoded

    @property
    def cdata(self):
        return utils.cdata(self.value)

    def __str__(self):
        if version_info < (3,):
            return self.__unicode__().encode("utf-8")
        else:
            return self.__unicode__()

    def __unicode__(self):
        return text_type(self.value)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from functools import partial

from mixbox import fields

import stix
from stix.common import Activity, Confidence, Statement
from stix.common.statement import StatementField
from stix.common.related import (GenericRelationshipList, RelatedCampaign,
    RelatedIncident, RelatedIndicator, RelatedPackageRefs, RelatedThreatActor,
    RelatedTTP)
from stix.common import vocabs
from stix.common.vocabs import VocabField, CampaignStatus
import stix.bindings.campaign as campaign_binding
from stix.common.structured_text import StructuredTextList
from stix.common.information_source import InformationSource
from stix.utils.deprecated import DeprecatedList


class AssociatedCampaigns(GenericRelationshipList):
    _namespace = "http://stix.mitre.org/Campaign-1"
    _binding = campaign_binding
    _binding_class = campaign_binding.AssociatedCampaignsType

    campaign = fields.TypedField("Associated_Campaign", RelatedCampaign, multiple=True, key_name="campaigns")


class Attribution(GenericRelationshipList):
    _namespace = "http://stix.mitre.org/Campaign-1"
    _binding = campaign_binding
    _binding_class = campaign_binding.AttributionType

    threat_actor = fields.TypedField("Attributed_Threat_Actor", RelatedThreatActor, multiple=True, key_name="threat_actors")


class RelatedIncidents(GenericRelationshipList):
    _namespace = "http://stix.mitre.org/Campaign-1"
    _binding = campaign_binding
    _binding_class = campaign_binding.RelatedIncidentsType

    incident = fields.TypedField("Related_Incident", RelatedIncident, multiple=True, key_name="incidents")


class RelatedIndicators(GenericRelationshipList):
    _namespace = "http://stix.mitre.org/Campaign-1"
    _binding = campaign_binding
    _binding_class = campaign_binding.RelatedIndicatorsType

    indicator = fields.TypedField(
        name="Related_Indicator",
        type_=RelatedIndicator,
        multiple=True,
        key_name="indicators",
        listfunc=partial(DeprecatedList, type=RelatedIndicator)
    )


class RelatedTTPs(GenericRelationshipList):
    _namespace = "http://stix.mitre.org/Campaign-1"
    _binding = campaign_binding
    _binding_class = campaign_binding.RelatedTTPsType

    ttp = fields.TypedField("Related_TTP", RelatedTTP, multiple=True, key_name="ttps")


class Names(stix.EntityList):
    _namespace = "http://stix.mitre.org/Campaign-1"
    _binding = campaign_binding
    _binding_class = campaign_binding.NamesType

    name = VocabField("Name", multiple=True, key_name="names")

    @classmethod
    def _dict_as_list(cls):
        return False


class Campaign(stix.BaseCoreComponent):
    """Implementation of the STIX Campaign.

    Args:
        id_ (optional): An identifier. If ``None``, a value will be generated
            via ``mixbox.idgen.create_id()``. If set, this will unset the
            ``idref`` property.
        idref (optional): An identifier reference. If set this will unset the
            ``id_`` property.
        timestamp (optional): A timestamp value. Can be an instance of
            ``datetime.datetime`` or ``str``.
        description: A description of the purpose or intent of this object.
        short_description: A short description of the intent
            or purpose of this object.
        title: The title of this object.

    """
    _binding = campaign_binding
    _binding_class = _binding.CampaignType
    _namespace = "http://stix.mitre.org/Campaign-1"
    _version = "1.2"
    _ALL_VERSIONS = ("1.0", "1.0.1", "1.1", "1.1.1", "1.2")
    _ID_PREFIX = 'campaign'

    descriptions = fields.TypedField("Description", StructuredTextList)
    activity = fields.TypedField("Activity", Activity, multiple=True)
    associated_campaigns = fields.TypedField("Associated_Campaigns", AssociatedCampaigns)
    attribution = fields.TypedField("Attribution", Attribution, multiple=True)
    confidence = fields.TypedField("Confidence", Confidence)
    # references = fields.TypedField("Reference", multiple=True)
    status = VocabField("Status", CampaignStatus)
    intended_effects = StatementField("Intended_Effect", Statement, vocab_type=vocabs.IntendedEffect, multiple=True, key_name="intended_effects")
    names = fields.TypedField("Names", Names)
    related_incidents = fields.TypedField("Related_Incidents", RelatedIncidents)
    related_indicators = fields.TypedField("Related_Indicators", RelatedIndicators)
    related_packages = fields.TypedField("Related_Packages", RelatedPackageRefs)
    related_ttps = fields.TypedField("Related_TTPs", RelatedTTPs)
    information_source = fields.TypedField("Information_Source", InformationSource)

    def __init__(self, id_=None, idref=None, timestamp=None, title=None,
                 description=None, short_description=None):

        super(Campaign, self).__init__(
            id_=id_,
            idref=idref,
            timestamp=timestamp,
            title=title,
            description=description,
            short_description=short_description
        )

        self.related_ttps = RelatedTTPs()
        self.related_incidents = RelatedIncidents()
        self.related_indicators = RelatedIndicators()
        self.related_packages = RelatedPackageRefs()


    def add_intended_effect(self, value):
        self.intended_effects.append(value)

    def add_activity(self, value):
        """Adds an :class:`.Activity` object to the :attr:`activity`
        collection.

        """
        self.activity.append(value)

    @property
    def description(self):
        """A single description about the contents or purpose of this object.

        Default Value: ``None``

        Note:
            If this object has more than one description set, this will return
            the description with the lowest ordinality value.

        Returns:
            An instance of :class:`.StructuredText`

        """
        return next(iter(self.descriptions), None)

    @description.setter
    def description(self, value):
        self.descriptions = StructuredTextList(value)

    def add_description(self, description):
        """Adds a description to the ``descriptions`` collection.

        This is the same as calling "foo.descriptions.add(bar)".

        """
        self.descriptions.add(description)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from stix.test import EntityTestCase
from stix.test.common import structured_text_tests

from stix.common import Activity


class ActivityTests(EntityTestCase, unittest.TestCase):
    klass = Activity
    _full_dict = {
        'date_time': "2014-01-31T06:14:46",
        'description': "And then something happened...",
    }


class ActivityMultiDescTests(EntityTestCase, unittest.TestCase):
    klass = Activity
    _full_dict = {
        'date_time': "2014-01-31T06:14:46",
        'description': structured_text_tests.StructuredTextListTests._full_dict
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import stix
import stix.utils as utils
import stix.bindings.stix_common as common_binding


DATE_PRECISION_VALUES = ("year", "month", "day")
TIME_PRECISION_VALUES = ("hour", "minute", "second")
DATETIME_PRECISION_VALUES = DATE_PRECISION_VALUES + TIME_PRECISION_VALUES


def validate_precision(instance, value):
    if value is None:
        return
    elif value in DATETIME_PRECISION_VALUES:
        return
    else:
        error = "The precision must be one of {0}. Received '{1}'"
        error = error.format(DATETIME_PRECISION_VALUES, value)
        raise ValueError(error)


class DateTimeWithPrecision(stix.Entity):
    _binding = common_binding
    _binding_class = _binding.DateTimeWithPrecisionType
    _namespace = 'http://stix.mitre.org/common-1'

    value = fields.DateTimeField("valueOf_", key_name="value")
    precision = fields.TypedField("precision", preset_hook=validate_precision)

    def __init__(self, value=None, precision='second'):
        super(DateTimeWithPrecision, self).__init__()
        self.value = value
        self.precision = precision

    def to_dict(self):
        if self.precision == 'second':
            return utils.dates.serialize_value(self.value)
        return super(DateTimeWithPrecision, self).to_dict()

    @classmethod
    def from_dict(cls, cls_dict):
        if not cls_dict:
            return None
        elif not isinstance(cls_dict, dict):
            obj = cls()
            obj.value = cls_dict
        else:
            obj = super(DateTimeWithPrecision, cls).from_dict(cls_dict)

        return obj



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# stdlib
from functools import partial

# mixbox
from mixbox import entities
from mixbox import fields

# stix
import stix
from stix import utils
from stix.ttp import TTP
from stix.common.kill_chains import KillChains
from stix.bindings import stix_core as core_binding

# deprecation warnings
from stix.utils.deprecated import IdrefDeprecatedList


class TTPs(stix.EntityList):
    _binding = core_binding
    _binding_class = _binding.TTPsType
    _namespace = 'http://stix.mitre.org/stix-1'

    ttps = fields.TypedField(
        name="TTP",
        type_=TTP,
        multiple=True,
        key_name="ttps",
        listfunc=partial(IdrefDeprecatedList, type=TTP)
    )

    kill_chains = fields.TypedField("Kill_Chains", KillChains)

    def __init__(self, ttps=None):
        super(TTPs, self).__init__(ttps)
        self.kill_chains = KillChains()

    def add_ttp(self, ttp):
        self.append(ttp)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import stix
import stix.bindings.stix_common as common_binding
import stix.utils

from .datetimewithprecision import DateTimeWithPrecision
from .structured_text import StructuredTextList


class Activity(stix.Entity):
    _binding = common_binding
    _binding_class = common_binding.ActivityType
    _namespace = 'http://stix.mitre.org/common-1'

    date_time = fields.TypedField("Date_Time", DateTimeWithPrecision)
    descriptions = fields.TypedField("Description", StructuredTextList)

    def __init__(self):
        super(Activity, self).__init__()
        self.descriptions = StructuredTextList()

    @property
    def description(self):
        """A single description about the contents or purpose of this object.

        Default Value: ``None``

        Note:
            If this object has more than one description set, this will return
            the description with the lowest ordinality value.

        Returns:
            An instance of
            :class:`.StructuredText`

        """
        return next(iter(self.descriptions), None)

    @description.setter
    def description(self, value):
        self.descriptions = StructuredTextList(value)

    def add_description(self, description):
        """Adds a description to the ``descriptions`` collection.

        This is the same as calling "foo.descriptions.add(bar)".

        """
        self.descriptions.add(description)




# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from stix.test import EntityTestCase

from stix.common import Identity


class IdentityTests(EntityTestCase, unittest.TestCase):
    klass = Identity
    _full_dict = {
        'id': "foo",       
        'name': "Me",
        'related_identities': {
            'identities': [
                {
                    'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
                    'identity': {'id': "foo1"}
                },
                {
                    'confidence': {'value': 'Sorta, kinda confident'},
                    'identity': {'id': "foo2"}
                }
            ]
        }
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.
import itertools
import contextlib
import collections
from sys import version_info

from mixbox import fields

import stix
import stix.utils as utils
import stix.bindings.stix_common as stix_common_binding
from mixbox.vendor.six import text_type


#: Default ordinality value for StructuredText.
DEFAULT_ORDINALITY = 1


class StructuredText(stix.Entity):
    """Used for storing descriptive text elements.

    Attributes:
        id_: An id for the text element, typically used for controlled
            structure xpath selectors.
        value: The text value of this object.
        structuring_format: The format of the text. For example, ``html5``.
    """

    _binding = stix_common_binding
    _binding_class = _binding.StructuredTextType
    _namespace = 'http://stix.mitre.org/common-1'

    id_ = fields.IdField("id")
    ordinality = fields.TypedField("ordinality")
    value = fields.TypedField("valueOf_", key_name="value")
    structuring_format = fields.TypedField("structuring_format")


    def __init__(self, value=None, ordinality=None):
        super(StructuredText, self).__init__()

        self.id_ = None
        self.value = value
        self.structuring_format = None
        self.ordinality = ordinality

    def is_plain(self):
        plain = (
            (not self.id_) and
            (not self.structuring_format) and
            (self.ordinality is None)
        )

        return plain

    def to_dict(self):
        """Converts this object into a dictionary representation.

        Note:
            If no properies or attributes are set other than ``value``,
            this will return a string.

        """
        # Return a plain string if there is no format specified.
        if self.is_plain():
            return self.value
        else:
            return super(StructuredText, self).to_dict()

    def __str__(self):
        """Returns a UTF-8 encoded string representation of the ``value``.

        """
        if version_info < (3,):
            return self.__unicode__().encode("utf-8")
        else:
            return self.__unicode__()

    def __unicode__(self):
        """Returns a ``unicode`` string representation of the ``value``.

        """
        return text_type(self.value)


@contextlib.contextmanager
def _unset_default(text):
    """Unsets the ordinality of the StructuredText object `text` if the
    ordinality is equal to the DEFAULT_ORDINALITY.

    The ordinaity will be returned to its original state after exiting the
    context manager.

    """
    ordinality = text.ordinality

    try:
        if ordinality == DEFAULT_ORDINALITY:
            text.ordinality = None  # Unset

        yield #  Return to caller
    finally:
        # Reset
        text.ordinality = ordinality


class StructuredTextList(stix.TypedCollection, collections.Sequence):
    """A sequence type used to store StructureText objects.

    Args:
        *args: A variable-length argument list which can contain single
            :class:`.StructuredText` objects or sequences of objects.

    """
    _treat_none_as_empty_list = True
    _contained_type = StructuredText
    _try_cast = True

    def __init__(self, *args):
        stix.TypedCollection.__init__(self, *args)

    def _initialize_inner(self, *args):
        # Check if it was initialized with args=None
        if not any(args):
            return

        for arg in args:
            if utils.is_sequence(arg):
                self.update(arg)
            else:
                self.add(arg)

    def with_id(self, id):
        """Returns a :class:`.StructuredText` object with a matching `id` or
        ``None`` if not found.

        """
        for text in self._inner:
            if text.id_ == id:
                return text

        # Not found. Return None.
        return None

    def reset(self):
        """Assigns sequential ordinality values to each of the sorted
        :class:`.StructuredText` objects, starting with ``1`` and ending
        at ``len(self)``.

        """
        for idx, item in enumerate(self.sorted, 1):
            item.ordinality = idx

    @property
    def sorted(self):
        """Returns a copy of the collection of internal
        :class:`.StructuredText` objects, sorted by their ``ordinality``.

        """
        return sorted(self._inner, key=lambda x: int(x.ordinality))

    @property
    def ordinalities(self):
        """Returns a sorted list of all the ``ordinality`` attribute
        values of the internal :class:`StructuredTex` objects.

        """
        return tuple(x.ordinality for x in self.sorted)

    @property
    def next_ordinality(self):
        """Returns the "+1" of the highest ordinality in the collection.

        """
        ords = self.ordinalities

        if not ords:
            return 1

        return ords[-1] + 1

    def __iter__(self):
        """Returns an iterator for the collection sorted by ordinality.

        """
        return iter(self.sorted)

    def __getitem__(self, key):
        """Returns the :class:`.StructuredText` object with a matching
        ordinality.

        Args:
            key: An ordinality value.

        Raises:
            KeyError: If `key` does not match the ordinality of any
                :class:`.StructuredText` object.

        """
        o = int(key)

        for item in self._inner:
            if item.ordinality == o:
                return item

        error = "No item found with an ordinality of {0}".format(o)
        raise KeyError(error)

    def __delitem__(self, key):
        """Removes the item with a given ordinality.

        Args:
            key: An ordinality value.

        Raises:
            KeyError: If the `key` does not match the ordinality for any object
                in the collection.

        """
        self._inner.remove(self[key])

    def __reversed__(self):
        """Yields the :class:`StructuredText` collection in descending order
        of their ordinalities.

        """
        for text in reversed(self.sorted):
            yield text

    def add(self, value):
        """Adds the :class:`.StructuredText` `value` to the collection.

        If `value` is not a :class:`.StructuredText` object, an attempt will
        be made to convert it to one.

        Note:
            If `value` does not have an ``ordinality`` set, one will be
            assigned. If `value` has an ordinality which matches one already
            in the collection, `value` will replace the existing item.

        Args:
            value: A :class:`.StructuredText` object.

        """
        if not self._is_valid(value):
            value = self._fix_value(value)

        if value.ordinality is None:
            value.ordinality = self.next_ordinality

        # Remove the existing item if there is one.
        with utils.ignored(KeyError):
            del self[value.ordinality]

        self._inner.append(value)

    def update(self, iterable):
        """Adds each item of `iterable` to the collection.

        Note:
            Any existing objects with conflicting ordinality values will be
            overwritten.

        Args:
            iterable: An iterable collection of :class:`.StructuredText` objects
                to add to this collection.

        """
        for item in iterable:
            self.add(item)

    def _shift(self, ordinality):
        """Increments the ordinality values on all objects in the collection
        that have an ordinality greater than or equal to `ordinality`.

        This is used in ``insert()`` operations.

        Note:
            This will only shift contiguous ordinalities, so if the collection
            contains the ordinaliities [1,2,6], then _shift(1) would result in
            [2,3,6] since 6 is not contiguous with [1,2].

        """
        to_shift = []

        for o in itertools.count(ordinality):
            try:
                to_shift.append(self[o])
            except KeyError:
                break

        for text in to_shift:
            text.ordinality += 1

    def insert(self, value):
        """Inserts `value` into the collection.

        If `value` has an ordinality which conflicts with an existing value,
        the existing value (and any contiguous values) will have their
        ordinality values incremented by one.

        """
        if not self._is_valid(value):
            value = self._fix_value(value)

        if value.ordinality is None:
            self.add(value)
        else:
            self._shift(value.ordinality)
            self._inner.append(value)

    def remove(self, value):
        """Removes the value from the collection.

        """
        self._inner.remove(value)

    def to_obj(self, ns_info=None):
        """Returns a binding object list for the StructuredTextList.

        If the list has a length of 1, and its member has an ordinality of 1,
        the ordinality will be unset.

        """
        #print "STList to_obj called"
        
        if not self:
            return []

        if len(self) > 1:
            return super(StructuredTextList, self).to_obj(ns_info=ns_info)

        # One item. Temporarily unset the ordinality if its the default value.
        text = self._inner[0]

        with _unset_default(text):
            l = [text.to_obj(ns_info=ns_info)]

        return l

    def to_list(self):
        """Returns a list of dictionary representations of the contained
        objects.

        An attempt is made to flatten out the returned list when there is only
        one item in the collection. This is to support backwards
        compatibility with previous versions of python-stix.

        * If the list repr has more than one item, return the list.
        * If there is only one item, inspect it.

          * If the item is not a dictionary, return it.
          * If its ``ordinality`` key has a corresponding value of ``1``, remove
            it from the dictionary since it's assumed if there is only one item.
          * After removing ``ordinality``, if the only key left is ``value``,
            just return the value of ``value`` (a string).

        """
        if not self:
            return []

        if len(self) > 1:
            return super(StructuredTextList, self).to_list()

        # One item. Temporarily unset the ordinality if its the default value.
        text = self._inner[0]

        # Temporarily unset the ordinality to create our dictionary
        with _unset_default(text):
            d = text.to_dict()

        return d

    to_dict = to_list




# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import contextlib
import functools
import itertools
import json
import warnings

import cybox.utils
from mixbox.binding_utils import ExternalEncoding
from mixbox.entities import NamespaceCollector
from mixbox.vendor.six import iteritems, text_type

from stix.utils import silence_warnings


@contextlib.contextmanager
def ctx_assert_warnings(self):
    """Context manager for verifying that a block of code has raised a
    warning.

    """
    with warnings.catch_warnings(record=True) as w:
        # Raise all warnings
        warnings.simplefilter('always')

        # Return to caller
        yield

        # Assert that a warning was raised.
        self.assertTrue(len(w) > 0)


def assert_warnings(func):
    """Test function decorator which asserts that a warning has been raised
    during the execution of the test.

    """
    @functools.wraps(func)
    def inner(*args, **kwargs):
        self = args[0]
        with ctx_assert_warnings(self):
            return func(*args, **kwargs)

    return inner


def round_trip_dict(cls, dict_):
    obj = cls.object_from_dict(dict_)
    dict2 = cls.dict_from_object(obj)

    api_obj = cls.from_dict(dict_)
    dict2 = cls.to_dict(api_obj)
    return dict2


def round_trip(o, output=False, list_=False):
    """ Performs all eight conversions to verify import/export functionality.

    1. cybox.Entity -> dict/list
    2. dict/list -> JSON string
    3. JSON string -> dict/list
    4. dict/list -> cybox.Entity
    5. cybox.Entity -> Bindings Object
    6. Bindings Object -> XML String
    7. XML String -> Bindings Object
    8. Bindings object -> cybox.Entity

    It returns the final object, so tests which call this function can check to
    ensure it was not modified during any of the transforms.
    """

    klass = o.__class__
    if output:
        print("Class: ", klass)
        print("-" * 40)

    # 1. cybox.Entity -> dict/list
    if list_:
        d = o.to_list()
    else:
        d = o.to_dict()

    # 2. dict/list -> JSON string
    json_string = json.dumps(d)

    if output:
        print(json_string)
        print("-" * 40)

    # Before parsing the JSON, make sure the cache is clear
    cybox.utils.cache_clear()

    # 3. JSON string -> dict/list
    d2 = json.loads(json_string)

    # 4. dict/list -> cybox.Entity
    if list_:
        o2 = klass.from_list(d2)
    else:
        o2 = klass.from_dict(d2)

    # 5. Entity -> Bindings Object
    ns_info = NamespaceCollector()
    xobj = o2.to_obj(ns_info=ns_info)

    try:
        # 6. Bindings Object -> XML String
        xml_string = o2.to_xml(encoding=ExternalEncoding)

        if not isinstance(xml_string, text_type):
            xml_string = xml_string.decode(ExternalEncoding)

    except KeyError as ex:
        print(str(ex))
        ns_info.finalize()
        print(ns_info.binding_namespaces)
        raise ex

    if output:
        print(xml_string)
        print("-" * 40)

    # Before parsing the XML, make sure the cache is clear
    cybox.utils.cache_clear()

    # 7. XML String -> Bindings Object
    xobj2 = klass._binding.parseString(xml_string)

    # 8. Bindings object -> cybox.Entity
    o3 = klass.from_obj(xobj2)

    return o3


class EntityTestCase(object):
    """A base class for testing STIX Entities"""

    def setUp(self):
        self.assertNotEqual(self.klass, None)
        self.assertNotEqual(self._full_dict, None)

    @silence_warnings
    def test_round_trip_full_dict(self):
        # Don't run this test on the base class
        if type(self) is EntityTestCase:
            return

        dict2 = round_trip_dict(self.klass, self._full_dict)
        self.maxDiff = None
        self.assertEqual(self._full_dict, dict2)

    def _combine(self, d):
        items = itertools.chain(
            iteritems(self._full_dict),
            iteritems(d)
        )

        return dict(items)

    @silence_warnings
    def test_round_trip_full(self):
        # Don't run this test on the base class
        if type(self) is EntityTestCase:
            return

        ent = self.klass.from_dict(self._full_dict)
        ent2 = round_trip(ent, output=True)

    @silence_warnings
    def _test_round_trip_dict(self, input):
        dict2 = round_trip_dict(self.klass, input)
        self.maxDiff = None
        self.assertEqual(input, dict2)

    @silence_warnings
    def _test_partial_dict(self, partial):
        d = self._combine(partial)
        self._test_round_trip_dict(d)


class TypedListTestCase(object):

    @silence_warnings
    def test_round_trip_rt(self):
        if type(self) is TypedListTestCase:
            return

        obj = self.klass.from_dict(self._full_dict)
        dict2 = obj.to_dict()
        self.assertEqual(self._full_dict, dict2)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import collections

from lxml import etree as etree_
import mixbox.xml


TypeInfo = collections.namedtuple("TypeInfo", ('ns', 'typename'))


def get_type_info(node):
    """Returns a ``TypeInfo`` object for `node`.

    This is accomplished by parsing the ``xsi:type`` attribute found on
    `node`.

    Args:
        node: An lxml.etree element object.

    Raises:
        KeyError: If `node` does not have an ``xsi:type`` attribute.

    """
    xsi_type = node.attrib[mixbox.xml.TAG_XSI_TYPE]
    typeinfo = xsi_type.split(":")

    if len(typeinfo) == 2:
        prefix, typename = typeinfo
    else:
        typename = typeinfo
        prefix = None

    ns = node.nsmap[prefix]
    return TypeInfo(ns=ns, typename=typename)


#: A mapping of namespace/type information to binding classes.
_BINDING_EXTENSION_MAP = {}


def add_extension(cls):
    """Adds the binding class `cls` to the ``_EXTENSION_MAP``.

    This enables the lookup and instantiation of classes during parse when
    ``xsi:type`` attributes are encountered.

    """
    typeinfo = TypeInfo(ns=cls.xmlns, typename=cls.xml_type)
    _BINDING_EXTENSION_MAP[typeinfo] = cls


def register_extension(cls):
    """Class decorator for registering a binding class as an implementation of
    an xml type.

    Classes must have ``xmlns`` and ``xml_type`` class attributes to be
    registered.

    """
    add_extension(cls)
    return cls


def lookup_extension(typeinfo, default=None):
    """Looks up the binding class for `typeinfo`, which is a namespace/typename
    pairing.

    Args:
        typeinfo: An lxml Element node or a stix.bindings.TypeInfo namedtuple.
        default: A binding class that will be returned if typeinfo is an
            Element without an xsi:type attribute.

    Returns:
        A binding class that has been registered for the namespace and typename
        found on `typeinfo`.

    """
    if not isinstance(typeinfo, TypeInfo):
        if has_xsi_type(typeinfo):
            typeinfo = get_type_info(typeinfo)
        elif default:
            return default

    if typeinfo in _BINDING_EXTENSION_MAP:
        return _BINDING_EXTENSION_MAP[typeinfo]

    fmt = "No class implemented or registered for XML type '{%s}%s'"
    error = fmt % (typeinfo.ns, typeinfo.typename)
    raise NotImplementedError(error)


def has_xsi_type(node):
    """Returns ``True`` if `node` does not have an xsi:type attribute.

    """
    return mixbox.xml.TAG_XSI_TYPE in node.attrib


__all__ = [
    'TypeInfo',
    'add_extension',
    'etree_',
    'get_type_info',
    'has_xsi_type',
    'lookup_extension',
    'register_extension',
]



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import stix
import stix.indicator.test_mechanism
from stix.common.vocabs import VocabField
from stix.common import EncodedCDATA, StructuredTextList, VocabString
from stix.indicator.test_mechanism import _BaseTestMechanism
import stix.bindings.extensions.test_mechanism.generic as generic_tm_binding


@stix.register_extension
class GenericTestMechanism(_BaseTestMechanism):
    _namespace = "http://stix.mitre.org/extensions/TestMechanism#Generic-1"
    _binding = generic_tm_binding
    _binding_class = _binding.GenericTestMechanismType
    _XSI_TYPE = "genericTM:GenericTestMechanismType"
    
    reference_location = fields.TypedField("reference_location")
    descriptions = fields.TypedField("Description", StructuredTextList)
    specification = fields.TypedField("Specification", EncodedCDATA)
    type_ = VocabField("Type")
    
    def __init__(self, id_=None, idref=None):
        super(GenericTestMechanism, self).__init__(id_=id_, idref=idref)
        self.descriptions = StructuredTextList()

    @property
    def description(self):
        """A single description about the contents or purpose of this object.

        Default Value: ``None``

        Note:
            If this object has more than one description set, this will return
            the description with the lowest ordinality value.

        Returns:
            An instance of
            :class:`.StructuredText`

        """
        return next(iter(self.descriptions), None)

    @description.setter
    def description(self, value):
        self.descriptions = StructuredTextList(value)

    def add_description(self, description):
        """Adds a description to the ``descriptions`` collection.

        This is the same as calling "foo.descriptions.add(bar)".

        """
        if self.descriptions is None:
            self.descriptions = StructuredTextList()
        self.descriptions.add(description)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# stdlib
import itertools

# external
from cybox.common import ObjectProperties
from mixbox.vendor.six import iteritems

# internal
from . import is_entity, is_entitylist, attr_name, is_sequence


def _is_skippable(owner, varname, varobj):
    if varname == "_fields" and isinstance(varobj, dict):
        return True

    if varname == "_parent" and isinstance(owner, ObjectProperties):
        return True

    if varname in ("__input_namespaces__", "__input_schemalocations__"):
        return True

    return False


def _iter_vars(obj):
    attrs = []

    if hasattr(obj, "__dict__"):
        attrs.append(iteritems(vars(obj)))

    if hasattr(obj, "_fields"):
        attrs.append(iteritems(obj._fields))

    return itertools.chain.from_iterable(attrs)


def iterwalk(obj):
    """Returns an generator which 'walks` the input `obj` model. Each
    iteration yields a stix.Entity or cybox.Entity instance.

    This is performed depth-first.

    """
    def yield_and_walk(item):
        if not is_entity(item):
            return

        yield item
        for descendant in iterwalk(item):
            yield descendant

    for varname, varobj in _iter_vars(obj):
        if _is_skippable(obj, varname, varobj):
            continue

        if is_sequence(varobj) and not is_entitylist(varobj):
            for item in varobj:
                for descendant in yield_and_walk(item):
                    yield descendant

            continue

        for descendant in yield_and_walk(varobj):
            yield descendant


def iterpath(obj, path=None):
    """Returns a generator which `walks` the input `obj` model. Each
    iteration yields a triple containing a list of ancestor nodes, the name
    of the field, and the field value.

    Example:
        An Indicator Title with a value of "Test" could be yielded as follows:
        ([STIXPackage, Indicators, Indicator], "title", "Test")

    This is performed depth-first.

    """
    def yield_and_descend(name, item):
        yield (path, attr_name(name), item)

        if item is None:
            return

        for path_info in iterpath(item, path):
            yield path_info

    if path is None:
        path = []

    path.append(obj)

    for varname, varobj in _iter_vars(obj):
        if _is_skippable(obj, varname, varobj):
            continue

        if varname == "_inner" and is_entitylist(obj):
            for item in varobj:
                for path_info in iterpath(item, path):
                    yield path_info
        elif is_sequence(varobj) and not is_entitylist(varobj):
            for item in varobj:
                for path_info in yield_and_descend(varname, item):
                    yield path_info

        else:
            for path_info in yield_and_descend(varname, varobj):
                yield path_info

    path.pop()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# mixbox
from mixbox import fields

# internal
import stix
import stix.bindings.exploit_target as exploit_target_binding
from stix.common.related import (
    GenericRelationshipList, RelatedCOA, RelatedExploitTarget,
    RelatedPackageRefs
)

# relative
from .vulnerability import Vulnerability, _Vulnerabilities  # noqa
from .weakness import Weakness, _Weaknesses  # noqa
from .configuration import Configuration  # noqa
from stix.common import InformationSource

class ExploitTarget(stix.BaseCoreComponent):
    """Implementation of STIX Exploit Target.

    Args:
        id_ (optional): An identifier. If ``None``, a value will be generated
            via ``mixbox.idgen.create_id()``. If set, this will unset the
            ``idref`` property.
        idref (optional): An identifier reference. If set this will unset the
            ``id_`` property.
        title (optional): A string title.
        timestamp (optional): A timestamp value. Can be an instance of
            ``datetime.datetime`` or ``str``.
        description (optional): A string description.
        short_description (optional): A string short description.

    """
    _binding = exploit_target_binding
    _binding_class = _binding.ExploitTargetType
    _namespace = "http://stix.mitre.org/ExploitTarget-1"
    _version = "1.2"
    _ALL_VERSIONS = ("1.0", "1.0.1", "1.1", "1.1.1", "1.2")
    _ID_PREFIX = 'et'

    potential_coas = fields.TypedField("Potential_COAs", type_="stix.exploit_target.PotentialCOAs")
    related_exploit_targets = fields.TypedField("Related_Exploit_Targets", type_="stix.exploit_target.RelatedExploitTargets")
    vulnerabilities = fields.TypedField("Vulnerability", Vulnerability, multiple=True, key_name="vulnerabilities")
    weaknesses = fields.TypedField("Weakness", Weakness, multiple=True, key_name="weaknesses")
    configuration = fields.TypedField("Configuration", Configuration, multiple=True)
    related_packages = fields.TypedField("Related_Packages", RelatedPackageRefs)
    information_source = fields.TypedField("Information_Source", InformationSource)

    def __init__(self, id_=None, idref=None, timestamp=None, title=None,
                 description=None, short_description=None):

        super(ExploitTarget, self).__init__(
            id_=id_,
            idref=idref,
            timestamp=timestamp,
            title=title,
            description=description,
            short_description=short_description
        )

        self.potential_coas = PotentialCOAs()
        self.related_exploit_targets = RelatedExploitTargets()
        self.related_packages = RelatedPackageRefs()

    def add_vulnerability(self, value):
        """Adds a vulnerability to the :attr:`vulnerabilities` list property.

        Note:
            If ``None`` is passed in no value is added 

        Args:
            value: A :class:`.Vulnerability` object..
            
        Raises:
            ValueError: if the `value` param is of type :class:`.Vulnerability`

        """
        self.vulnerabilities.append(value)
               
    def add_weakness(self, value):
        """Adds a weakness to the :attr:`weaknesses` list property.

        Note:
            If ``None`` is passed in no value is added 

        Args:
            value: A :class:`.Weakness` object.
            
        Raises: ValueError if the `value` param is of type :class:`.Weakness`

        """
        self.weaknesses.append(value)

    def add_configuration(self, value):
        """Adds a configuration to the :attr:`configurations` list property.

        Note:
            If ``None`` is passed in no value is added 

        Args:
            value: A configuration value.
            
        Raises:
            ValueError: If the `value` param is of type :class:`.Configuration`

        """
        self.configuration.append(value)


class PotentialCOAs(GenericRelationshipList):
    """
    A list of ``Potential_COA`` objects, defaults to empty array
    """
    _namespace = "http://stix.mitre.org/ExploitTarget-1"
    _binding = exploit_target_binding
    _binding_class = exploit_target_binding.PotentialCOAsType

    potential_coa = fields.TypedField("Potential_COA", RelatedCOA, multiple=True, key_name="coas")

    def __init__(self, coas=None, scope=None):
        super(PotentialCOAs, self).__init__(scope, coas)


class RelatedExploitTargets(GenericRelationshipList):
    """
    A list of ``RelatedExploitTargets`` objects, defaults to empty array
    """
    _namespace = "http://stix.mitre.org/ExploitTarget-1"
    _binding = exploit_target_binding
    _binding_class = exploit_target_binding.RelatedExploitTargetsType

    related_exploit_target = fields.TypedField("Related_Exploit_Target", RelatedExploitTarget, multiple=True, key_name="related_exploit_targets")

    def __init__(self, related_exploit_targets=None, scope=None):
        super(RelatedExploitTargets, self).__init__(scope, related_exploit_targets)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
import stix.bindings.incident as incident_binding
from mixbox import fields, entities

class ExternalID(stix.Entity):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding  = incident_binding
    _binding_class = incident_binding.ExternalIDType

    value = fields.TypedField("valueOf_", key_name="value")
    source = fields.TypedField("source")

    def __init__(self, value=None, source=None):
        super(ExternalID, self).__init__()
        self.value = value
        self.source = source



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.
from mixbox import fields

import stix
from stix.common import DateTimeWithPrecision
import stix.bindings.indicator as indicator_binding
from mixbox.entities import Entity

class ValidTime(Entity):
    _namespace = "http://stix.mitre.org/Indicator-2"
    _binding = indicator_binding
    _binding_class = _binding.ValidTimeType
    
    start_time = fields.TypedField("Start_Time", DateTimeWithPrecision)
    end_time = fields.TypedField("End_Time", DateTimeWithPrecision)
    
    def __init__(self, start_time=None, end_time=None):
        super(ValidTime, self).__init__()
        self.start_time = start_time
        self.end_time = end_time




# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# stdlib
import functools

# mixbox
from mixbox import fields
from mixbox import typedlist

# cybox
from cybox.core import Observable

# internal
import stix
import stix.bindings.stix_common as common_binding
import stix.bindings.stix_core as core_binding
import stix.bindings.report as report_binding

# deprecation warnings
from stix.utils import deprecated

# relative
from .vocabs import VocabField
from .information_source import InformationSource
from .confidence import Confidence


ALLOWED_SCOPE = ('inclusive', 'exclusive')


def validate_scope(instance, value):
    if not value:
        return
    elif value in ALLOWED_SCOPE:
        return
    else:
        msg = "Scope must be one of {0}. Received '{1}'"
        msg = msg.format(ALLOWED_SCOPE, value)
        raise ValueError(msg)


class GenericRelationship(stix.Entity):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = common_binding.GenericRelationshipType

    confidence = fields.TypedField("Confidence", Confidence)
    information_source = fields.TypedField("Information_Source", InformationSource)
    relationship = VocabField("Relationship")

    def __init__(self, confidence=None, information_source=None, relationship=None):
        super(GenericRelationship, self).__init__()

        self.confidence = confidence
        self.information_source = information_source
        self.relationship = relationship


class RelatedPackageRef(GenericRelationship):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = common_binding.RelatedPackageRefType

    idref = fields.IdrefField("idref")
    timestamp = fields.DateTimeField("timestamp")

    def __init__(self, idref=None, timestamp=None, confidence=None,
                 information_source=None, relationship=None):

        super(RelatedPackageRef, self).__init__(
            confidence=confidence,
            information_source=information_source,
            relationship=relationship
        )

        self.idref = idref
        self.timestamp = timestamp


class GenericRelationshipEntity(stix.Entity):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = _binding.GenericRelationshipListType
    
    _ALLOWED_SCOPE = ('inclusive', 'exclusive')
    
    scope = fields.TypedField("scope")
    
    def __init__(self, scope=None, *args):

        super(GenericRelationshipEntity, self).__init__(*args)
        self.scope = scope
    
    def __nonzero__(self):
        return (
            super(GenericRelationshipList, self).__nonzero__() or
            bool(self.scope)
        )


class GenericRelationshipList(stix.EntityList):
    """Base class for concrete GenericRelationshipList types.

    Note:
        Subclasses must supply exactly one multiple TypedField.
    """

    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = _binding.GenericRelationshipListType

    scope = fields.TypedField("scope", preset_hook=validate_scope)

    def __init__(self, scope=None, *args):
        super(GenericRelationshipList, self).__init__(*args)
        self.scope = scope

    def __nonzero__(self):
        return (super(GenericRelationshipList, self).__nonzero__() or
                bool(self.scope))

    @classmethod
    def _dict_as_list(cls):
        return False


class _RelatedPackageList(typedlist.TypedList):
    def __init__(self, *args):
        super(_RelatedPackageList, self).__init__(type=RelatedPackageRef, *args)

    def _fix_value(self, value):
        from stix.core import STIXPackage

        if isinstance(value, STIXPackage) and value.id_:
            return RelatedPackageRef(idref=value.id_, timestamp=value.timestamp)

        fmt = ("Cannot add type '{0}' to RelatedPackageRefs collection. "
               "Expected RelatedPackageRef or STIXPackage")
        error = fmt.format(type(value))
        raise TypeError(error)

    def _is_valid(self, value):
        deprecated.warn(value)
        return super(_RelatedPackageList, self)._is_valid(value)


class RelatedPackageRefs(stix.EntityList):
    _namespace = 'http://stix.mitre.org/common-1'
    _binding = common_binding
    _binding_class = common_binding.RelatedPackageRefsType

    package = fields.TypedField(
        name="Package_Reference",
        type_=RelatedPackageRef,
        multiple=True,
        key_name="packages",
        listfunc=_RelatedPackageList
    )

    @classmethod
    def _dict_as_list(cls):
        return False


class _BaseRelated(GenericRelationship):
    """A base class for related types.

    This class is not a real STIX type and should not be directly instantiated.

    Note:
        Subclasses must supply a TypedField named `item`!
    """

    item = None  # override in subclass.

    def __init__(self, item=None, confidence=None,
                 information_source=None, relationship=None):

        super(_BaseRelated, self).__init__(
            confidence,
            information_source,
            relationship
        )

        self.item = item


class RelatedCampaign(_BaseRelated):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = common_binding.RelatedCampaignType

    # _BaseRelated requires an "item" field.
    item = fields.TypedField("Campaign", type_="stix.campaign.Campaign")


class RelatedCOA(_BaseRelated):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = common_binding.RelatedCourseOfActionType

    # _BaseRelated requires an "item" field.
    item = fields.TypedField("Course_Of_Action", type_="stix.coa.CourseOfAction")


class RelatedExploitTarget(_BaseRelated):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = common_binding.RelatedExploitTargetType

    # _BaseRelated requires an "item" field.
    item = fields.TypedField("Exploit_Target", type_="stix.exploit_target.ExploitTarget")


class RelatedIdentity(_BaseRelated):
    _namespace = 'http://stix.mitre.org/common-1'
    _binding = common_binding
    _binding_class = common_binding.RelatedIdentityType

    # _BaseRelated requires an "item" field.
    item = fields.TypedField("Identity", type_="stix.common.identity.Identity", factory="stix.common.identity.IdentityFactory")


class RelatedIncident(_BaseRelated):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = common_binding.RelatedIncidentType

    # _BaseRelated requires an "item" field.
    item = fields.TypedField("Incident", type_="stix.incident.Incident")

class RelatedIndicator(_BaseRelated):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = common_binding.RelatedIndicatorType

    # _BaseRelated requires an "item" field.
    item = fields.TypedField("Indicator", type_="stix.indicator.Indicator")


class RelatedObservable(_BaseRelated):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = common_binding.RelatedObservableType

    # _BaseRelated requires an "item" field.
    item = fields.TypedField("Observable", type_=Observable)

class RelatedThreatActor(_BaseRelated):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = common_binding.RelatedThreatActorType

    # _BaseRelated requires an "item" field.
    item = fields.TypedField("Threat_Actor", type_="stix.threat_actor.ThreatActor")


class RelatedTTP(_BaseRelated):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = common_binding.RelatedTTPType

    # _BaseRelated requires an "item" field.
    item = fields.TypedField("TTP", type_="stix.ttp.TTP")


class RelatedPackage(_BaseRelated):
    _namespace = "http://stix.mitre.org/stix-1"
    _binding = core_binding
    _binding_class = core_binding.RelatedPackageType

    # _BaseRelated requires an "item" field.
    item = fields.TypedField("Package", type_="stix.core.STIXPackage", preset_hook=deprecated.field)


class RelatedReport(_BaseRelated):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = common_binding.RelatedReportType

    # _BaseRelated requires an "item" field.
    item = fields.TypedField("Report", type_="stix.report.Report")


class RelatedCampaignRef(_BaseRelated):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = _binding.RelatedCampaignReferenceType

    # _BaseRelated requires an "item" field.
    item = fields.TypedField("Campaign", type_="stix.common.CampaignRef")


class RelatedPackages(GenericRelationshipList):
    _namespace = 'http://stix.mitre.org/stix-1'
    _binding = core_binding
    _binding_class = core_binding.RelatedPackagesType

    related_package = fields.TypedField("Related_Package", RelatedPackage, multiple=True, key_name="related_packages")


class RelatedReports(GenericRelationshipList):
    _namespace = 'http://stix.mitre.org/Report-1'
    _binding = report_binding
    _binding_class = report_binding.RelatedReportsType

    related_report = fields.TypedField("Related_Report", RelatedReport, multiple=True, key_name="related_reports")



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# external
from mixbox import fields

# internal
import stix
import stix.bindings.stix_common as common_binding

# relative
from .vocabs import VocabField


class Names(stix.EntityList):
    _namespace = 'http://stix.mitre.org/common-1'
    _binding = common_binding
    _binding_class = _binding.NamesType

    name = VocabField("Name", multiple=True)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import stix.bindings.ttp as ttp_binding
from stix.common.related import GenericRelationshipList, RelatedExploitTarget


class ExploitTargets(GenericRelationshipList):
    _namespace = "http://stix.mitre.org/TTP-1"
    _binding = ttp_binding
    _binding_class = _binding.ExploitTargetsType

    exploit_target = fields.TypedField("Exploit_Target", RelatedExploitTarget, multiple=True, key_name="exploit_targets")



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from stix.test import EntityTestCase, TypedListTestCase, assert_warnings
from stix.test import data_marking_test
from stix.test.common import information_source_test, related_test

from stix.core import STIXPackage
import stix.exploit_target as et
from stix.exploit_target import weakness, vulnerability, configuration

class CVSSVectorTests(EntityTestCase, unittest.TestCase):
    klass = vulnerability.CVSSVector

    _full_dict = {
        'overall_score': "9.3",
        'base_score': "8.3",
        'base_vector': "(AV:N/AC:L/Au:N/C:N/I:N/A:P)",
        'temporal_score': "7.7",
        'temporal_vector': "(E:U/RL:T/RC:C)",
        'environmental_score': "2.0",
        'environmental_vector': "(CPD:ND/TD:ND)",
    }


class AffectedSoftwareTests(EntityTestCase, unittest.TestCase):
    klass = vulnerability.AffectedSoftware

    _full_dict = {
        'scope': 'inclusive',
        'affected_software': [
            related_test.RelatedObservableTests._full_dict
        ]
    }


class VulnerabilityTests(EntityTestCase, unittest.TestCase):
    klass = vulnerability.Vulnerability

    _full_dict = {
        'is_known': False,
        'is_publicly_acknowledged': True,
        'title': "CVE-2012-0158",
        'description': "Vulnerability Description",
        'short_description': "MSCOMCTL.OCX Memory Corruption",
        'cvss_score': CVSSVectorTests._full_dict,
        'discovered_datetime': {
            'value': '2010-02-21T00:00:00',
            'precision': 'day',
        },
        'published_datetime': {
            'value': '2010-03-01T00:00:00',
            'precision': 'month',
        },
        'affected_software': AffectedSoftwareTests._full_dict,
        'references': ['foo','bar']
    }


class VulnerabilitiesTests(TypedListTestCase, unittest.TestCase):
    klass = vulnerability._Vulnerabilities

    _full_dict = [
        VulnerabilityTests._full_dict
    ]


class PotentialCOAsTests(EntityTestCase, unittest.TestCase):
    klass = et.PotentialCOAs

    _full_dict = {
        'scope': 'inclusive',
        'coas': [
            related_test.RelatedCOATests._full_dict
        ]
    }


class RelatedExploitTargetsTests(EntityTestCase, unittest.TestCase):
    klass = et.RelatedExploitTargets

    _full_dict = {
        'scope': 'inclusive',
        'related_exploit_targets': [
            related_test.RelatedExploitTargetTests._full_dict
        ]
    }

class WeaknessTests(EntityTestCase, unittest.TestCase):
    klass = weakness.Weakness

    _full_dict = {
        'description': "Deadlock",
        'cwe_id': "CWE-833",
    }


class WeaknessesTests(TypedListTestCase, unittest.TestCase):
    klass = weakness._Weaknesses

    _full_dict = [
        WeaknessTests._full_dict
    ]



class ConfigurationTests(EntityTestCase, unittest.TestCase):
    klass = configuration.Configuration

    _full_dict =  {
        'description': "The 'Games' features should be configured"
                       "correctly.",
        'short_description': "Games feature",
        'cce_id': "CCE-18880-5",
    }


class ExploitTargetTests(EntityTestCase, unittest.TestCase):
    klass = et.ExploitTarget
    _full_dict = {
        'id': 'example:test-1',
        #idref omitted since it should not have both an ID and an IDREF.
        'timestamp': "2014-04-01T03:17:45",
        'version': '1.1',
        'title': "ExploitTarget 1",
        'description': "This is a long description about an ExploitTarget",
        'short_description': "an ExploitTarget",
        'vulnerabilities': VulnerabilitiesTests._full_dict,
        'weaknesses': WeaknessesTests._full_dict,
        'configuration': [ConfigurationTests._full_dict],
        'potential_coas': PotentialCOAsTests._full_dict,
        'information_source': information_source_test.InformationSourceTests._full_dict,
        'handling': data_marking_test.MarkingTests._full_dict,
        'related_exploit_targets': RelatedExploitTargetsTests._full_dict,
        'related_packages': related_test.RelatedPackageRefsTests._full_dict
    }

    def test_add_description(self):
        o1 = self.klass()
        o2 = self.klass()

        o1.add_description("Test")
        o2.descriptions.add("Test")

        self.assertEqual(
            o1.descriptions.to_dict(),
            o2.descriptions.to_dict()
        )

    def test_add_short_description(self):
        o1 = self.klass()
        o2 = self.klass()

        o1.add_short_description("Test")
        o2.short_descriptions.add("Test")

        self.assertEqual(
            o1.short_descriptions.to_dict(),
            o2.short_descriptions.to_dict()
        )

    @assert_warnings
    def test_deprecated_related_packages(self):
        e = et.ExploitTarget()
        e.related_packages.append(STIXPackage())
        self.assertEqual(len(e.related_packages), 1)

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from __future__ import absolute_import

from mixbox import fields

import stix
import stix.utils as utils
import stix.bindings.stix_common as common_binding

from .datetimewithprecision import validate_precision
from .confidence import Confidence
from .structured_text import StructuredTextList
from .vocabs import VocabField, HighMediumLow
import mixbox
from stix.common.vocabs import VocabString

class Statement(stix.Entity):
    _namespace = 'http://stix.mitre.org/common-1'
    _binding = common_binding
    _binding_class = common_binding.StatementType

    # Fields
    timestamp = fields.DateTimeField("timestamp")
    timestamp_precision = fields.TypedField("timestamp_precision", preset_hook=validate_precision)
    value = VocabField("Value", VocabString)
    descriptions = fields.TypedField("Description", StructuredTextList)
    confidence = fields.TypedField("Confidence", Confidence)
    source = fields.TypedField("Source", type_="stix.common.InformationSource")

    def __init__(self, value=None, timestamp=None, description=None,
                 source=None):
        super(Statement, self).__init__()

        self.timestamp = timestamp or utils.dates.now()
        self.timestamp_precision = "second"
        self.value = value
        self.descriptions = StructuredTextList(description)
        self.source = source
        self.confidence = None

    @property
    def description(self):
        """A single description about the contents or purpose of this object.

        Default Value: ``None``

        Note:
            If this object has more than one description set, this will return
            the description with the lowest ordinality value.

        Returns:
            An instance of
            :class:`.StructuredText`

        """
        return next(iter(self.descriptions), None)

    @description.setter
    def description(self, value):
        self.descriptions = value

    def add_description(self, description):
        """Adds a description to the ``descriptions`` collection.

        This is the same as calling "foo.descriptions.add(bar)".

        """
        self.descriptions.add(description)


class StatementField(mixbox.fields.TypedField):
    def __init__(self, *args, **kwargs):
        self._vocab_type = kwargs.pop("vocab_type")
        super(StatementField, self).__init__(*args, **kwargs)
        self.type_ = Statement

    def _clean(self, value):
        if value is None:
            return None
        elif isinstance(value, Statement):
            return value
        elif isinstance(value, stix.common.VocabString):
            return Statement(value)
        else:
            vocabklass = self._vocab_type
            return Statement(vocabklass(value)) 



import unittest

from stix.test import EntityTestCase

from stix.extensions.marking.tlp import TLPMarkingStructure


class TLPMarkingStructureTests(EntityTestCase, unittest.TestCase):
    klass = TLPMarkingStructure
    _full_dict = {
        'marking_model_name': 'TLP',
        'marking_model_ref': "http://www.us-cert.gov/tlp/",
        'color': "RED",
        'xsi:type': "tlpMarking:TLPMarkingStructureType",
    }


if __name__ == "__main__":
    unittest.main()




# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# mixbox
from mixbox import fields

# stix
import stix
import stix.bindings.ttp as ttp_binding

from .malware_instance import MalwareInstance, MalwareInstanceFactory
from .exploit import Exploit
from .attack_pattern import AttackPattern

class Behavior(stix.Entity):
    _binding = ttp_binding
    _binding_class = _binding.BehaviorType
    _namespace = "http://stix.mitre.org/TTP-1"

    malware_instances = fields.TypedField("Malware", type_="stix.ttp.behavior.MalwareInstances", key_name="malware_instances")
    attack_patterns = fields.TypedField("Attack_Patterns", type_="stix.ttp.behavior.AttackPatterns")
    exploits = fields.TypedField("Exploits", type_="stix.ttp.behavior.Exploits")

    def __init__(self, malware_instances=None, attack_patterns=None, exploits=None):
        super(Behavior, self).__init__()
        self.malware_instances = malware_instances or MalwareInstances()
        self.attack_patterns = attack_patterns or AttackPatterns()
        self.exploits = exploits or Exploits()


    def add_malware_instance(self, malware):
        self.malware_instances.append(malware)

    def add_attack_pattern(self, attack_pattern):
        self.attack_patterns.append(attack_pattern)

    def add_exploit(self, exploit):
        self.exploits.append(exploit)


class Exploits(stix.EntityList):
    _namespace = "http://stix.mitre.org/TTP-1"
    _contained_type = Exploit
    _binding = ttp_binding
    _binding_class = _binding.ExploitsType

    exploit = fields.TypedField("Exploit", Exploit, multiple=True, key_name="exploits")


class MalwareInstances(stix.EntityList):
    _namespace = "http://stix.mitre.org/TTP-1"
    _binding = ttp_binding
    _binding_class = _binding.MalwareType

    malware_instance = fields.TypedField("Malware_Instance", MalwareInstance, multiple=True, factory=MalwareInstanceFactory, key_name="malware_instances")


class AttackPatterns(stix.EntityList):
    _namespace = "http://stix.mitre.org/TTP-1"
    _binding = ttp_binding
    _binding_class = _binding.AttackPatternsType

    attack_pattern = fields.TypedField("Attack_Pattern", AttackPattern, multiple=True, key_name="attack_patterns")



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# stdlib
import unittest

# external
import lxml.etree
from mixbox.vendor.six import StringIO

# internal
import mixbox.namespaces
from stix.core import STIXPackage
from stix.utils import silence_warnings


class NamespaceInfoTests(unittest.TestCase):

    @silence_warnings
    def test_user_provided_ns(self):
        """Test that user-provided namespaces are serialized.

        """
        p = STIXPackage()

        TEST_PREFIX = 'test'
        TEST_NS = 'a:unit:test'
        NEW_STIX_PREFIX = 'newstix'
        NEW_STIX_NS = "http://stix.mitre.org/stix-1"

        test_dict = {
            TEST_NS: TEST_PREFIX,
            NEW_STIX_NS: NEW_STIX_PREFIX
        }

        # Parse the exported document and make sure that the namespaces
        # made it through the serialization process.
        xml = p.to_xml(ns_dict=test_dict)
        e = lxml.etree.XML(xml)
        self.assertEqual(e.nsmap.get(TEST_PREFIX), TEST_NS)
        self.assertEqual(e.nsmap.get(NEW_STIX_PREFIX), NEW_STIX_NS)

    @silence_warnings
    def test_duplicate_ns_prefix(self):
        """Test that duplicate namespace prefix mappings raise errors.

        """
        p = STIXPackage()
        bad = {'bad:ns': 'stix'}  # 'stix' is already default ns prefix

        self.assertRaises(
            mixbox.namespaces.DuplicatePrefixError,
            p.to_xml,
            ns_dict=bad
        )

        # Build a valid stix document that has a default namespace remapped
        # to another namespace. We remap 'stixCommon' to a bogus ns here.
        xml = (
            """<stix:STIX_Package
                    xmlns:stixCommon="THISISGONNABEPROBLEM"
                    xmlns:stix="http://stix.mitre.org/stix-1"
                    xmlns:stixVocabs="http://stix.mitre.org/default_vocabularies-1"
                    version="1.2"
                    timestamp="2015-04-09T14:22:25.620831+00:00">
                <stix:STIX_Header>
                    <stix:Description>A unit test</stix:Description>
                </stix:STIX_Header>
            </stix:STIX_Package>"""
        )

        sio = StringIO(xml)
        p = STIXPackage.from_xml(sio)

        # Exporting should raise an error.
        self.assertRaises(
            mixbox.namespaces.DuplicatePrefixError,
            p.to_xml
        )


    @silence_warnings
    def test_parsed_namespaces(self):
        """Test that non-default namespaces make it through the parse-serialize
        process.

        """
        xml = (
            """<stix:STIX_Package
                    xmlns:TEST="a:test"
                    xmlns:FOO="a:foo"
                    xmlns:BAR="a:bar"
                    xmlns:cybox="http://cybox.mitre.org/cybox-2"
                    xmlns:cyboxCommon="http://cybox.mitre.org/common-2"
                    xmlns:cyboxVocabs="http://cybox.mitre.org/default_vocabularies-2"
                    xmlns:example="http://example.com"
                    xmlns:stix="http://stix.mitre.org/stix-1"
                    xmlns:stixCommon="http://stix.mitre.org/common-1"
                    xmlns:stixVocabs="http://stix.mitre.org/default_vocabularies-1"
                    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                    id="example:Package-e2454ee8-e59c-43ac-a085-46ae4516fd6e"
                    version="1.2"
                    timestamp="2015-04-09T14:22:25.620831+00:00"/>"""
        )

        sio = StringIO(xml)
        p = STIXPackage.from_xml(sio)

        serialized = p.to_xml()
        e = lxml.etree.XML(serialized)
        self.assertEqual(e.nsmap.get('TEST'), 'a:test')
        self.assertEqual(e.nsmap.get('FOO'), 'a:foo')
        self.assertEqual(e.nsmap.get('BAR'), 'a:bar')


if __name__ == "__main__":
    unittest.main()


# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# stdlib
import json
import collections
import itertools
from sys import version_info

# mixbox
from mixbox import idgen
from mixbox import entities
from mixbox import fields
from mixbox import binding_utils
from mixbox import namespaces
from mixbox.vendor.six import StringIO, iteritems, itervalues, text_type, binary_type

# internal
from . import utils

def _override(*args, **kwargs):
    raise NotImplementedError()


class Entity(entities.Entity):
    """Base class for all classes in the STIX API."""
    _namespace = None
    _XSI_TYPE = None

    def _set_var(self, klass, try_cast=True, arg=None, **kwargs):
        """Sets an instance property value.

        * If the input value is ``None``, the property is set to ``None``.
        * If the input value is an instance of `klass`, the property is set
          the input value.
        * If the input value is not an instance of `klass` and `try_cast` is
          ``True``, an attempt will be made to cast the input value to an
          instance of `klass`.

        Args:
            klass: The expected input value class.
            try_cast: If ``True`` attempt to cast the input value to `klass`
                if it is not an instance of `klass`.
            arg: The __init__ parameter name to use when casting the input
                value to `klass`. E.g., StructuredText(value=input), the `arg`
                is `value`. If ``None``, it is assumed that the first
                __init__ parameter will accept the value.
            **kwargs: The field name and value. The field name is the key
                and the field value is the value.

        """
        name, item = next(iteritems(kwargs))
        attr = utils.private_name(name)  # 'title' => '_title'

        if item is None:
            setattr(self, attr, None)
        elif isinstance(item, klass):
            setattr(self, attr, item)
        elif try_cast:
            promoted = utils.cast_var(item, klass, arg=arg)
            setattr(self, attr, promoted)
        else:
            error = "'{0}' expects an instance of {1}. Received: {2}."
            error = error.format(name, klass, type(item))
            raise TypeError(error)

    def _set_vocab(self, klass=None, **kwargs):
        """Sets a controlled vocabulary property value.

        * If the input value is ``None``, the property is set to ``None``.
        * If the input value is an instance of ``VocabString``, the property
          is set to the input value.
        * If the input value is not an instance of ``VocabString``, an attempt
          will be made to cast the input to an instance of `klass`. If `klass`
          is ``None``, ``VocabString`` will be used.

        Args:
            klass: The VocabString impl to cast the input value to. If ``None``
                ``VocabString`` will be assumed.
            **kwargs: The field name, value pair. The field name is the key
                and the field value is the value.

        """
        from stix.common import VocabString

        klass = klass or VocabString
        item  = next(itervalues(kwargs))

        if isinstance(item, VocabString):
            self._set_var(VocabString, **kwargs)
        else:
            self._set_var(klass, **kwargs)

    def to_xml(self, include_namespaces=True, include_schemalocs=False,
               ns_dict=None, schemaloc_dict=None, pretty=True,
               auto_namespace=True, encoding='utf-8'):
        """Serializes a :class:`Entity` instance to an XML string.

        The default character encoding is ``utf-8`` and can be set via the
        `encoding` parameter. If `encoding` is ``None``, a string (unicode in
        Python 2, str in Python 3) is returned.

        Args:
            auto_namespace: Automatically discover and export XML namespaces
                for a STIX :class:`Entity` instance.
            include_namespaces: Export namespace definitions in the output
                XML. Default is ``True``.
            include_schemalocs: Export ``xsi:schemaLocation`` attribute
                in the output document. This will attempt to associate
                namespaces declared in the STIX document with schema locations.
                If a namespace cannot be resolved to a schemaLocation, a
                Python warning will be raised. Schemalocations will only be
                exported if `include_namespaces` is also ``True``.
            ns_dict: Dictionary of XML definitions (namespace is key, alias is
                value) to include in the exported document. This must be
                passed in if `auto_namespace` is ``False``.
            schemaloc_dict: Dictionary of XML ``namespace: schema location``
                mappings to include in the exported document. These will
                only be included if `auto_namespace` is ``False``.
            pretty: Pretty-print the XML.
            encoding: The output character encoding. Default is ``utf-8``. If
                `encoding` is set to ``None``, a string (unicode in Python 2,
                str in Python 3) is returned.

        Returns:
            An XML string for this
            :class:`Entity` instance. Default character encoding is ``utf-8``.

        """

        from mixbox.entities import NamespaceCollector

        if (not auto_namespace) and (not ns_dict):
            raise Exception(
                "Auto-namespacing was disabled but ns_dict was empty "
                "or missing."
            )

        ns_info = NamespaceCollector()

        obj = self.to_obj(ns_info=ns_info if auto_namespace else None)

        ns_info.finalize(ns_dict=ns_dict, schemaloc_dict=schemaloc_dict)

        if auto_namespace:
            obj_ns_dict = ns_info.binding_namespaces
        else:
            obj_ns_dict = dict(
                itertools.chain(
                    iteritems(ns_info.binding_namespaces),
                    iteritems(namespaces.get_full_ns_map())
                )
            )

        namespace_def = ""
        if include_namespaces:
            delim = "\n\t" if pretty else " "
            xmlns = ns_info.get_xmlns_string(delim)
            namespace_def += (delim + xmlns)
            if include_schemalocs:
                schemaloc = ns_info.get_schema_location_string(delim)
                namespace_def += (delim + schemaloc)

        with binding_utils.save_encoding(encoding):
            sio = StringIO()
            obj.export(
                sio.write,                    # output buffer
                0,                            # output level
                obj_ns_dict,                  # namespace dictionary
                pretty_print=pretty,          # pretty printing
                namespacedef_=namespace_def   # namespace/schemaloc def string
            )

        # Ensure that the StringIO buffer is unicode
        s = text_type(sio.getvalue())

        if encoding:
            return s.encode(encoding)

        return s

    def walk(self):
        return utils.walk.iterwalk(self)

    def find(self, id_):
        """Searches the children of a :class:`Entity` implementation for an
        object with an ``id_`` property that matches `id_`.

        """
        if not id_:
            return

        for entity in self.walk():
            if id_ == getattr(entity, "id_", None):
                return entity


class EntityList(entities.EntityList, Entity):
    def to_xml(self, *args, **kwargs):
        return Entity.to_xml(self, *args, **kwargs)


class TypedCollection(object):
    """Abstract base class for non-STIX collections of entities.

    See also:
        TypedList

    """
    _contained_type = _override

    def __init__(self, *args):
        self._inner = []
        self._initialize_inner(*args)

    def _initialize_inner(self, *args):
        """Must be overridden by subclass.

        """
        raise NotImplementedError()

    def __len__(self):
        return len(self._inner)

    def __nonzero__(self):
        return bool(self._inner)

    def _is_valid(self, value):
        """Check if this is a valid object to add to the list."""
        # Subclasses can override this function, but if it becomes common,
        # it's probably better to use self._contained_type.istypeof(value)
        return isinstance(value, self._contained_type)

    def _fix_value(self, value):
        """Attempt to coerce value into the correct type.

        Subclasses can override this function.
        """
        try:
            new_value = self._contained_type(value)
        except Exception:
            error = "Can't put '{0}' ({1}) into a {2}. Expected a {3} object."
            error = error.format(
                value,                  # Input value
                type(value),            # Type of input value
                type(self),             # Type of collection
                self._contained_type    # Expected type of input value
            )
            raise ValueError(error)

        return new_value

    def to_obj(self, ns_info=None):
        #print "TypedCollection to_obj called"
        return [x.to_obj(ns_info=ns_info) for x in self]

    def to_list(self):
        return [h.to_dict() for h in self]

    @classmethod
    def from_obj(cls, obj_list, contained_type=None):
        if obj_list is None:
            return None

        if not contained_type:
            contained_type = cls._contained_type

        if not utils.is_sequence(obj_list):
            obj_list = [obj_list]

        items = (contained_type.from_obj(x) for x in obj_list)
        return cls(items)

    @classmethod
    def from_list(cls, list_repr, contained_type=None):

        if not list_repr:
            return cls()

        if isinstance(list_repr, dict) or not utils.is_sequence(list_repr):
            list_repr = [list_repr]

        if not contained_type:
            contained_type = cls._contained_type

        items = (contained_type.from_dict(x) for x in list_repr)
        return cls(items)

    from_dict = from_list
    to_dict = to_list

    @classmethod
    def object_from_dict(cls, entity_dict):
        """Convert from dict representation to object representation."""
        return cls.from_dict(entity_dict).to_obj()

    @classmethod
    def dict_from_object(cls, entity_obj):
        """Convert from object representation to dict representation."""
        return cls.from_obj(entity_obj).to_dict()
    
    @classmethod
    def istypeof(cls, obj):
        """Check if `cls` is the type of `obj`

        In the normal case, as implemented here, a simple isinstance check is
        used. However, there are more complex checks possible. For instance,
        EmailAddress.istypeof(obj) checks if obj is an Address object with
        a category of Address.CAT_EMAIL
        """
        return isinstance(obj, cls)


class TypedList(TypedCollection, collections.MutableSequence):
    def __init__(self, *args):
        TypedCollection.__init__(self, *args)

    def _initialize_inner(self, *args):
        # Check if it was initialized with args=None
        if not any(args):
            return

        for arg in args:
            if utils.is_sequence(arg):
                self.extend(arg)
            else:
                self.append(arg)

    def __getitem__(self, key):
        return self._inner.__getitem__(key)

    def __setitem__(self, key, value):
        if not self._is_valid(value):
            value = self._fix_value(value)
        self._inner.__setitem__(key, value)

    def __delitem__(self, key):
        self._inner.__delitem__(key)

    def __len__(self):
        return len(self._inner)

    def insert(self, idx, value):
        if not value:
            return
        if not self._is_valid(value):
            value = self._fix_value(value)
        self._inner.insert(idx, value)


def _validate_version(instance, value):
    if value:
        utils.check_version(instance._ALL_VERSIONS, value)


class BaseCoreComponent(Entity):
    _ALL_VERSIONS = ()
    _ID_PREFIX = None

    title = fields.TypedField("Title")
    id_ = fields.IdField("id")
    idref = fields.IdrefField("idref")
    descriptions = fields.TypedField("Description", type_="stix.common.StructuredTextList", )
    short_descriptions = fields.TypedField("Short_Description", type_="stix.common.StructuredTextList")
    version = fields.TypedField("version", preset_hook=_validate_version)
    timestamp = fields.DateTimeField("timestamp")
    handling = fields.TypedField("Handling", type_="stix.data_marking.Marking")


    def __init__(self, id_=None, idref=None, timestamp=None, title=None,
                 description=None, short_description=None):
        from stix.common import StructuredTextList

        super(BaseCoreComponent, self).__init__()

        self.id_ = id_ or idgen.create_id(self._ID_PREFIX)
        self.idref = idref
        self.title = title
        self.descriptions = StructuredTextList(description)
        self.short_descriptions = StructuredTextList(short_description)

        if timestamp:
            self.timestamp = timestamp
        else:
            self.timestamp = utils.dates.now() if not idref else None

    @property
    def description(self):
        """A single description about the contents or purpose of this object.

        Default Value: ``None``

        Note:
            If this object has more than one description set, this will return
            the description with the lowest ordinality value.

        Returns:
            An instance of :class:`.StructuredText`
        """
        return next(iter(self.descriptions), None)

    @description.setter
    def description(self, value):
        self.descriptions = value

    def add_description(self, description):
        """Adds a description to the ``descriptions`` collection.

        This is the same as calling "foo.descriptions.add(bar)".
        """
        self.descriptions.add(description)

    @property
    def short_description(self):
        """A single short description about the contents or purpose of this
        object.

        Default Value: ``None``

        Note:
            If this object has more than one short description set, this will
            return the description with the lowest ordinality value.

        Returns:
            An instance of :class:`.StructuredText`
        """
        return next(iter(self.short_descriptions), None)

    @short_description.setter
    def short_description(self, value):
        self.short_descriptions = value

    def add_short_description(self, description):
        """Adds a description to the ``short_descriptions`` collection.

        This is the same as calling "foo.short_descriptions.add(bar)".
        """
        self.short_descriptions.add(description)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
from stix.xmlconst import TAG_STIX_PACKAGE

import mixbox.parser
# Import these from mixbox for backward compatibility
from mixbox.parser import (UnknownVersionError, UnsupportedVersionError,
                           UnsupportedRootElementError)

# Alias for backwards compatibility
UnsupportedRootElement = UnsupportedRootElementError


class EntityParser(mixbox.parser.EntityParser):

    def supported_tags(self):
        return [TAG_STIX_PACKAGE]

    def get_version(self, root):
        return root.attrib.get('version')

    def supported_versions(self, tag=TAG_STIX_PACKAGE):
        return stix.supported_stix_version()

    def get_entity_class(self, tag=TAG_STIX_PACKAGE):
        return stix.core.STIXPackage



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
import stix.bindings.incident as incident_binding
from mixbox import entities, fields

class LossEstimation(stix.Entity):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding  = incident_binding
    _binding_class = incident_binding.LossEstimationType

    iso_currency_code = fields.TypedField("iso_currency_code")
    amount = fields.TypedField("amount")

    def __init__(self):
        super(LossEstimation, self).__init__()



#!/usr/bin/env python
# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

"""
Demonstrates the methods for adding related Campaign references to an
Indicator object.
"""

from stix.core import STIXPackage
from stix.common import CampaignRef
from stix.campaign import Campaign
from stix.indicator import Indicator


def main():
    # Build Campaign instances
    camp1 = Campaign(title='Campaign 1')
    camp2 = Campaign(title='Campaign 2')

    # Build a CampaignRef object, setting the `idref` to the `id_` value of
    # our `camp2` Campaign object.
    campaign_ref = CampaignRef(idref=camp2.id_)

    # Build an Indicator object.
    i = Indicator()

    # Add CampaignRef object pointing to `camp2`.
    i.add_related_campaign(campaign_ref)

    # Add Campaign object, which gets promoted into an instance of
    # CampaignRef type internally. Only the `idref` is set.
    i.add_related_campaign(camp1)

    # Build our STIX Package and attach our Indicator and Campaign objects.
    package = STIXPackage()
    package.add_indicator(i)
    package.add_campaign(camp1)
    package.add_campaign(camp2)

    # Print!
    print package.to_xml()

if __name__ == "__main__":
    main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import stix
from stix.data_marking import MarkingStructure
import stix.bindings.extensions.marking.simple_marking as simple_marking_binding


@stix.register_extension
class SimpleMarkingStructure(MarkingStructure):
    _binding = simple_marking_binding
    _binding_class = simple_marking_binding.SimpleMarkingStructureType
    _namespace = 'http://data-marking.mitre.org/extensions/MarkingStructure#Simple-1'
    _XSI_TYPE = "simpleMarking:SimpleMarkingStructureType"

    statement = fields.TypedField("Statement")

    def __init__(self, statement=None):
        super(SimpleMarkingStructure, self).__init__()
        self.statement = statement
